import logging
import os

import yaml

from django.apps import apps
from django.conf import settings
from django.utils.encoding import force_str
from django.utils.functional import Promise
from django.utils.translation import gettext_lazy as _

from mayan.apps.common.serialization import yaml_dump, yaml_load

from .exceptions import SettingsException, SettingsExceptionRevert

logger = logging.getLogger(name=__name__)


class SettingMetaclass(type):
    def __call__(cls, namespace, global_name, **kwargs):
        try:
            instance = namespace.get_setting(global_name=global_name)
        except KeyError:
            instance = super().__call__(
                namespace=namespace, global_name=global_name, **kwargs
            )
        finally:
            return instance


class Setting(metaclass=SettingMetaclass):
    @staticmethod
    def deserialize_value(value):
        return yaml_load(stream=value)

    @staticmethod
    def express_promises(value):
        """
        Walk all the elements of a value and force promises to text.
        """
        if isinstance(value, (list, tuple)):
            return [
                Setting.express_promises(item) for item in value
            ]
        elif isinstance(value, Promise):
            return force_str(s=value)
        else:
            return value

    @staticmethod
    def serialize_value(value):
        result = yaml_dump(
            allow_unicode=True, data=Setting.express_promises(value=value),
            default_flow_style=False
        )
        # safe_dump returns bytestrings.
        # Disregard the last 3 dots that mark the end of the YAML document.
        if force_str(s=result).endswith('...\n'):
            result = result[:-4]

        return result

    def __init__(
        self, namespace, global_name, default, choices=None, help_text=None,
        is_path=False, post_edit_function=None, validation_function=None
    ):
        self.choices = choices
        self.default = default
        self.environment_variable = False
        self.global_name = global_name
        self.has_load_error = False
        self.help_text = help_text
        self.loaded = False
        self.namespace = namespace
        self.post_edit_function = post_edit_function
        self.validation_function = validation_function
        self.value_raw_new = None

        self.namespace.setting_dict[self.global_name] = self
        self.namespace.cluster.setting_dict[self.global_name] = self

    def __str__(self):
        return str(self.global_name)

    def do_cache_invalidate(self):
        self.loaded = False

    def do_migrate(self):
        self.namespace.do_migrate(setting=self)

    def do_post_edit_function_call(self):
        if self.post_edit_function:
            try:
                self.post_edit_function(setting=self)
            except Exception as exception:
                raise SettingsException(
                    'Unable to execute setting post update function '
                    'for setting "{}". Verify the value of the setting or '
                    'rollback to the previous known working configuration '
                    'file.'.format(self.global_name)
                ) from exception

    def do_value_cache(self, global_name=None, default_override=None):
        global_name = global_name or self.global_name

        environment_value = os.environ.get(
            'MAYAN_{}'.format(global_name)
        )
        if environment_value:
            self.environment_variable = True
            try:
                self.value_raw = yaml_load(stream=environment_value)
            except yaml.YAMLError as exception:
                self.has_load_error = True

                if settings.SETTINGS_IGNORE_ERRORS:
                    logger.error(
                        'Error interpreting environment variable: %s with '
                        'value: %s; %s', global_name, environment_value,
                        exception
                    )
                    self.value_raw = self.default
                else:
                    raise type(exception)(
                        'Error interpreting environment variable: {} with '
                        'value: {}; {}'.format(
                            global_name, environment_value, exception
                        )
                    )
        else:
            try:
                # Try the config file.
                configuration_file_content = self.namespace.cluster.get_configuration_file_content()
                self.value_raw = configuration_file_content[global_name]
            except KeyError:
                try:
                    # Try the Django settings variable.
                    self.value_raw = getattr(
                        settings, global_name
                    )
                except AttributeError:
                    # Finally set to the default value.
                    if default_override:
                        self.value_raw = default_override
                    else:
                        self.value_raw = self.default
            else:
                # Found in the config file, try to migrate the value.
                self.do_migrate()

        if self.validation_function:
            self.value_raw = self.validation_function(
                raw_value=self.value_raw, setting=self
            )

        self.value_yaml = Setting.serialize_value(value=self.value_raw)
        self.loaded = True

    def get_has_load_error(self):
        return self.has_load_error

    get_has_load_error.short_description = _(message='Has errors')
    get_has_load_error.help_text = _(
        message='Indicates that this setting was not loaded correctly. '
        'Settings with errors revert to their default value.'
    )

    def get_value_choices(self):
        return self.choices

    get_value_choices.short_description = _(message='Choices')
    get_value_choices.help_text = _(
        message='Possible values allowed for this setting.'
    )

    def do_value_raw_set(self, raw_value):
        self.value = Setting.serialize_value(value=raw_value)
        self.loaded = True

    def do_value_raw_validate(self, raw_value):
        if self.validation_function:
            return self.validation_function(
                raw_value=raw_value, setting=self
            )

    def do_value_revert(self):
        UpdatedSetting = apps.get_model(
            app_label='smart_settings', model_name='UpdatedSetting'
        )

        if not self.get_has_value_new():
            raise SettingsExceptionRevert(
                _(
                    message='Cannot revert setting. Setting value has not been '
                    'updated.'
                )
            )

        updated_setting = UpdatedSetting.objects.get(
            global_name=self.global_name
        )

        self.do_value_set(value=updated_setting.value_old)
        updated_setting.delete()

    def do_value_set(self, value):
        UpdatedSetting = apps.get_model(
            app_label='smart_settings', model_name='UpdatedSetting'
        )

        raw_value = Setting.deserialize_value(value=value)

        self.value_raw_new = raw_value

        if self.value_raw_new != self.value:
            updated_setting, created = UpdatedSetting.objects.update_or_create(
                global_name=self.global_name, defaults={
                    'value_new': self.value_raw_new,
                    'value_old': self.value
                }
            )
        else:
            queryset = UpdatedSetting.objects.filter(
                global_name=self.global_name
            )
            queryset.delete()

    def get_default(self):
        return Setting.serialize_value(value=self.default)

    get_default.short_description = _(message='Default')

    def get_has_value_new(self):
        UpdatedSetting = apps.get_model(
            app_label='smart_settings', model_name='UpdatedSetting'
        )

        queryset = UpdatedSetting.objects.filter(
            global_name=self.global_name
        )

        return queryset.exists()

    get_has_value_new.short_description = _(message='Modified')
    get_has_value_new.help_text = _(
        message='The value of this setting being modified since the last restart.'
    )

    def get_is_overridden(self):
        return self.environment_variable

    get_is_overridden.short_description = _(message='Overridden')
    get_is_overridden.help_text = _(
        message='The value of the setting is being overridden by an environment '
        'variable.'
    )

    def get_value_current(self):
        UpdatedSetting = apps.get_model(
            app_label='smart_settings', model_name='UpdatedSetting'
        )

        has_value_new = self.get_has_value_new()

        if has_value_new:
            updated_setting = UpdatedSetting.objects.get(
                global_name=self.global_name
            )

            return Setting.serialize_value(value=updated_setting.value_new)
        else:
            return self.serialized_value

    @property
    def pk(self):
        """
        Compatibility property for views that expect model instances.
        """
        return self.global_name

    @property
    def serialized_value(self):
        """
        YAML serialize value of the setting.
        Used for UI display.
        """
        if not self.loaded:
            self.do_value_cache()

        return self.value_yaml

    @property
    def value(self):
        if not self.loaded:
            self.do_value_cache()

        return self.value_raw

    @value.setter
    def value(self, value):
        # value is in YAML format.
        self.value_yaml = value
        self.value_raw = Setting.deserialize_value(value=self.value_yaml)



from django.apps import apps
from django.db import models
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

from mayan.apps.events.classes import EventType


class WorkflowManager(models.Manager):
    def launch_for(self, document, user=None):
        for workflow_template in document.document_type.workflows.all():
            if workflow_template.auto_launch:
                workflow_template.launch_for(document=document, user=user)


class WorkflowTransitionTriggerEventManager(models.Manager):
    def check_triggers(self, action):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        WorkflowInstance = apps.get_model(
            app_label='document_states', model_name='WorkflowInstance'
        )
        WorkflowTransition = apps.get_model(
            app_label='document_states', model_name='WorkflowTransition'
        )

        queryset_triggered_transitions = WorkflowTransition.objects.filter(
            trigger_events__event_type__name=action.verb
        )

        if isinstance(action.target, Document):
            queryset_workflow_instances = WorkflowInstance.objects.filter(
                workflow__transitions__in=queryset_triggered_transitions,
                document=action.target
            ).distinct()
        elif isinstance(action.action_object, Document):
            queryset_workflow_instances = WorkflowInstance.objects.filter(
                workflow__transitions__in=queryset_triggered_transitions,
                document=action.action_object
            ).distinct()
        else:
            queryset_workflow_instances = WorkflowInstance.objects.none()

        queryset_workflow_instances = queryset_workflow_instances.exclude(
            Q(workflow__ignore_completed=True) & Q(state_active__final=True)
        )

        for workflow_instance in queryset_workflow_instances:
            # Select the first transition that is valid for this workflow
            # state.
            queryset_valid_transitions = queryset_triggered_transitions & workflow_instance.get_queryset_valid_transitions()

            if queryset_valid_transitions.exists():
                event_type_label = EventType.get_label(id=action.verb)

                workflow_instance.do_transition(
                    comment=_(message='Event trigger: %s') % event_type_label,
                    transition=queryset_valid_transitions.first()
                )


class ValidWorkflowInstanceManager(models.Manager):
    def get_queryset(self):
        return models.QuerySet(
            model=self.model, using=self._db
        ).filter(document__in_trash=False)



import logging
from pathlib import Path

from django import apps
from django.utils.log import DEFAULT_LOGGING

from .settings import (
    setting_logging_disable_color_formatter, setting_logging_enable,
    setting_logging_handlers, setting_logging_level,
    setting_logging_log_file_path
)

logger = logging.getLogger(name=__name__)


class LoggingAppConfigMixin:
    def ready(self):
        super().ready()

        if setting_logging_enable.value:
            logging_configuration = DEFAULT_LOGGING.copy()

            mayan_loggin_configuration = {
                'version': 1,
                'disable_existing_loggers': False,
                'formatters': {
                    'mayan_intermediate': {
                        'format': '%(name)s <%(process)d> [%(levelname)s] "%(funcName)s() line %(lineno)d %(message)s"'
                    },
                    'mayan_logfile': {
                        'format': '%(asctime)s %(name)s <%(process)d> [%(levelname)s] "%(funcName)s() line %(lineno)d %(message)s"'
                    }
                },
                'handlers': {
                    'console': {
                        'class': 'logging.StreamHandler',
                        'formatter': 'mayan_intermediate',
                        'level': 'DEBUG'
                    }
                }
            }

            if not setting_logging_disable_color_formatter.value:
                mayan_loggin_configuration['formatters'][
                    'mayan_intermediate'
                ]['()'] = 'mayan.apps.logging.formatters.ColorFormatter'

            logging_configuration.update(mayan_loggin_configuration)

            # Convert to list to it mutable
            handlers = list(setting_logging_handlers.value)

            if 'logfile' in handlers:
                path = Path(setting_logging_log_file_path.value)
                try:
                    path.touch()
                except (FileNotFoundError, PermissionError):
                    # The path's folder do not exists or we lack
                    # permission to write the log file.
                    handlers.remove('logfile')
                else:
                    logging_configuration['handlers']['logfile'] = {
                        'backupCount': 5,
                        'class': 'logging.handlers.RotatingFileHandler',
                        'filename': setting_logging_log_file_path.value,
                        'formatter': 'mayan_logfile',
                        'maxBytes': 65535
                    }

            loggers = {}

            # Django loggers
            for key, value in logging_configuration['loggers'].items():
                value['level'] = setting_logging_level.value

            # Mayan apps loggers
            for project_app in apps.apps.get_app_configs():
                loggers[project_app.name] = {
                    'handlers': handlers,
                    'propagate': True,
                    'level': setting_logging_level.value
                }

            logging_configuration['loggers'] = loggers

            logging.config.dictConfig(config=logging_configuration)



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.links import Link
from mayan.apps.user_management.link_conditions import (
    condition_user_is_not_super_user
)
from mayan.apps.user_management.permissions import permission_user_edit

from .icons import icon_impersonate_start, icon_logout, icon_password_change
from .permissions import permission_users_impersonate


def _condition_user_has_usable_password_and_can_change_password(user):
    if user.is_authenticated:
        return user.has_usable_password() and not user.user_options.block_password_change
    else:
        return False


def condition_user_has_usable_password_and_can_change_password(context, resolved_object):
    user = context['request'].user

    return _condition_user_has_usable_password_and_can_change_password(
        user=user
    )


def condition_user_has_usable_password_and_can_change_password_and_is_not_admin(context, resolved_object):
    return _condition_user_has_usable_password_and_can_change_password(
        user=resolved_object
    ) and condition_user_is_not_super_user(
        context=context, resolved_object=resolved_object
    )


link_logout = Link(
    html_extra_classes='non-ajax', icon=icon_logout, text=_(message='Logout'),
    view='authentication:logout_view'
)
link_password_change = Link(
    condition=condition_user_has_usable_password_and_can_change_password,
    icon=icon_password_change, text=_(message='Change password'),
    view='authentication:password_change_view'
)
link_user_impersonate_form_start = Link(
    icon=icon_impersonate_start,
    permission=permission_users_impersonate, text=_(message='Impersonate user'),
    view='authentication:user_impersonate_form_start'
)
link_user_impersonate_start = Link(
    args='object.id', condition=condition_user_is_not_super_user, icon=icon_impersonate_start,
    permission=permission_users_impersonate, text=_(message='Impersonate'),
    view='authentication:user_impersonate_start'
)
link_user_multiple_set_password = Link(
    icon=icon_password_change, permission=permission_user_edit,
    text=_(message='Set password'), view='authentication:user_multiple_set_password'
)
link_user_set_password = Link(
    args='object.id', condition=condition_user_has_usable_password_and_can_change_password_and_is_not_admin,
    icon=icon_password_change, permission=permission_user_edit,
    text=_(message='Set password'), view='authentication:user_set_password'
)



from collections import deque
import functools

import elasticsearch
from elasticsearch import Elasticsearch, helpers
from elasticsearch.dsl import Search

from mayan.settings.literals import DEFAULT_ELASTICSEARCH_PASSWORD

from ...exceptions import (
    DynamicSearchBackendException, DynamicSearchValueTransformationError
)
from ...search_backends import SearchBackend
from ...search_fields import SearchFieldVirtualAllFields
from ...search_models import SearchModel

from .literals import (
    DEFAULT_ELASTICSEARCH_HOSTS, DEFAULT_ELASTICSEARCH_INDICES_NAMESPACE,
    DEFAULT_ELASTICSEARCH_INDICES_NAMESPACE_TEST,
    DEFAULT_ELASTICSEARCH_POINT_IN_TIME_KEEP_ALIVE,
    DEFAULT_ELASTICSEARCH_SEARCH_PAGE_SIZE,
    DJANGO_TO_ELASTICSEARCH_FIELD_MAP, INDEX_NAME_DELIMITER,
    MAXIMUM_API_ATTEMPT_COUNT
)


class ElasticsearchSearchBackend(SearchBackend):
    feature_reindex = True
    field_type_mapping = DJANGO_TO_ELASTICSEARCH_FIELD_MAP

    def __init__(
        self, client_kwargs=None,
        indices_namespace=DEFAULT_ELASTICSEARCH_INDICES_NAMESPACE,
        search_page_size=DEFAULT_ELASTICSEARCH_SEARCH_PAGE_SIZE,
        point_in_time_keep_alive=DEFAULT_ELASTICSEARCH_POINT_IN_TIME_KEEP_ALIVE,
        **kwargs

    ):
        super().__init__(**kwargs)

        self.indices_namespace = indices_namespace
        self.point_in_time_keep_alive = point_in_time_keep_alive
        self.search_page_size = search_page_size

        self.client_kwargs = client_kwargs or {
            'basic_auth': ('elastic', DEFAULT_ELASTICSEARCH_PASSWORD),
            'hosts': DEFAULT_ELASTICSEARCH_HOSTS,
            'verify_certs': False
        }

        if self._test_mode:
            self.indices_namespace = DEFAULT_ELASTICSEARCH_INDICES_NAMESPACE_TEST

        self._client = Elasticsearch(**self.client_kwargs)

    def do_search_execute(self, index_name, search):
        model = self._get_model_for_index(index_name=index_name)
        self._client.indices.refresh(index=index_name)

        pk_field = model._meta.pk

        point_in_time = self._client.open_point_in_time(
            index=index_name, keep_alive=self.point_in_time_keep_alive
        )
        pit_id = point_in_time['id']

        base = (
            Search(using=self._client)
            .extra(
                pit={
                    'id': pit_id, 'keep_alive': self.point_in_time_keep_alive
                },
                size=self.search_page_size,
                track_total_hits=False
            )
            .sort('_shard_doc')
            .source(False)
        )

        query_dict = search.to_dict().get('query')
        if query_dict:
            base = base.update_from_dict(
                {'query': query_dict}
            )

        search_after = None

        try:
            while True:
                if search_after is None:
                    search_new = base
                else:
                    search_new = search_new.extra(search_after=search_after)

                response = search_new.execute()

                if not response or not len(response):
                    break

                for entry in response:
                    result_id = entry.meta.id
                    yield pk_field.to_python(value=result_id)

                search_after = response[-1].meta.sort
        except elasticsearch.exceptions.NotFoundError as exception:
            raise DynamicSearchBackendException(
                'Index not found. Make sure the search engine '
                'was properly initialized or upgraded if '
                'it already existed.'
            ) from exception
        finally:
            self._client.close_point_in_time(id=pit_id)

    def _get_index_name(self, search_model):
        return '{}{}{}'.format(
            self.indices_namespace, INDEX_NAME_DELIMITER,
            search_model.full_name
        )

    @functools.lru_cache(maxsize=256)
    def _get_model_for_index(self, index_name):
        indices_namespace, search_model_name = index_name.split(
            INDEX_NAME_DELIMITER
        )
        search_model = SearchModel.get(name=search_model_name)
        model = search_model.model

        return model

    @functools.cache
    def _get_search_model_index_mappings(self, search_model):
        mappings = {}

        field_map = self.get_resolved_field_type_map(
            search_model=search_model
        )
        for field_name, search_field_data in field_map.items():
            mappings[field_name] = {
                'type': search_field_data['field'].name
            }

            if 'analyzer' in search_field_data:
                mappings[field_name]['analyzer'] = search_field_data['analyzer']

        return mappings

    def _get_status(self):
        result = []

        title = 'Elasticsearch search model indexing status'
        title_length = len(title)
        result.append(title)
        result.append('=' * title_length)

        self.refresh()

        for search_model in SearchModel.all():
            index_name = self._get_index_name(search_model=search_model)
            try:
                index_stats = self._client.count(index=index_name)
            except elasticsearch.exceptions.NotFoundError:
                index_stats = {}

            count = index_stats.get('count', 'None')

            result.append(
                '{}: {}'.format(search_model.label, count)
            )

        return '\n'.join(result)

    def _initialize(self):
        self._update_mappings()

    def _search(
        self, search_field, query_type, value, is_quoted_value=False,
        is_raw_value=False
    ):
        self.do_query_type_verify(
            query_type=query_type, search_field=search_field
        )

        index_name = self._get_index_name(
            search_model=search_field.search_model
        )

        if isinstance(search_field, SearchFieldVirtualAllFields):
            seen = set()

            for search_field in search_field.field_composition:
                try:
                    search_field_query = query_type.resolve_for_backend(
                        is_quoted_value=is_quoted_value,
                        is_raw_value=is_raw_value, search_backend=self,
                        search_field=search_field, value=value
                    )
                except DynamicSearchValueTransformationError:
                    """Skip the search field."""
                else:
                    if search_field_query is not None:
                        index_name = self._get_index_name(
                            search_model=search_field.search_model
                        )

                        search = Search(index=index_name, using=self._client)
                        search = search.filter(search_field_query)

                        result = self.do_search_execute(
                            index_name=index_name, search=search
                        )
                        for item in result:
                            if item not in seen:
                                seen.add(item)
                                yield item
            else:
                return
        else:
            search = Search(index=index_name, using=self._client)

            try:
                search_field_query = query_type.resolve_for_backend(
                    is_quoted_value=is_quoted_value,
                    is_raw_value=is_raw_value, search_backend=self,
                    search_field=search_field, value=value
                )
            except DynamicSearchValueTransformationError:
                return
            else:
                if search_field_query is None:
                    return
                else:
                    search = search.filter(search_field_query)

                    yield from self.do_search_execute(
                        index_name=index_name, search=search
                    )

    def _update_mappings(self, search_model=None):
        if search_model:
            search_models = (search_model,)
        else:
            search_models = SearchModel.all()

        for search_model in search_models:
            index_name = self._get_index_name(search_model=search_model)

            try:
                self._client.indices.delete(index=index_name)
            except elasticsearch.exceptions.NotFoundError:
                """
                Non fatal, might be that this is the first time
                the method is executed. Proceed.
                """

            mappings = self._get_search_model_index_mappings(
                search_model=search_model
            )

            try:
                self._client.indices.create(
                    index=index_name,
                    mappings={'properties': mappings}
                )
            except elasticsearch.exceptions.RequestError:
                try:
                    self._client.indices.put_mapping(
                        index=index_name, properties=mappings
                    )
                except elasticsearch.exceptions.RequestError:
                    """
                    There are mapping changes that were not allowed.
                    Example: Text to Keyword.
                    Boot up regardless and allow user to reindex to delete
                    old indices.
                    """

    def deindex_instance(self, instance):
        search_model = SearchModel.get_for_model(instance=instance)
        index_name = self._get_index_name(search_model=search_model)
        self._client.delete(id=instance.pk, index=index_name)

    def index_instance(
        self, instance, exclude_model=None, exclude_kwargs=None
    ):
        search_model = SearchModel.get_for_model(instance=instance)

        document = search_model.populate(
            exclude_kwargs=exclude_kwargs, exclude_model=exclude_model,
            instance=instance, search_backend=self
        )
        index_name = self._get_index_name(search_model=search_model)
        self._client.index(
            document=document, id=instance.pk, index=index_name
        )

    def index_instances(self, search_model, id_list):
        index_name = self._get_index_name(search_model=search_model)

        def generate_actions():
            queryset = search_model.get_queryset()
            queryset = queryset.filter(pk__in=id_list)

            for instance in queryset:
                kwargs = search_model.populate(
                    search_backend=self, instance=instance
                )
                kwargs['_id'] = kwargs['id']

                yield kwargs

        action = generate_actions()
        bulk_indexing_generator = helpers.streaming_bulk(
            actions=action, client=self._client, index=index_name,
            yield_ok=False
        )

        deque(iterable=bulk_indexing_generator, maxlen=0)

    def refresh(self):
        attempt_count = 0
        search_model_index = 0
        search_models = SearchModel.all()

        while True:
            search_model = search_models[search_model_index]
            index_name = self._get_index_name(search_model=search_model)

            try:
                self._client.indices.refresh(index=index_name)
            except elasticsearch.exceptions.NotFoundError as exception:
                attempt_count += 1

                if attempt_count > MAXIMUM_API_ATTEMPT_COUNT:
                    raise DynamicSearchBackendException(
                        'Refresh attempt count exceeded the maximum'
                        ' of `{}`.'.format(
                            MAXIMUM_API_ATTEMPT_COUNT
                        )
                    ) from exception
            else:
                attempt_count = 0
                search_model_index += 1
                if search_model_index >= len(search_models):
                    break

    def reset(self, search_model=None):
        self.tear_down(search_model=search_model)
        self._update_mappings(search_model=search_model)

    def tear_down(self, search_model=None):
        if search_model:
            search_models = (search_model,)
        else:
            search_models = SearchModel.all()

        for search_model in search_models:
            index_name = self._get_index_name(search_model=search_model)
            try:
                self._client.indices.delete(index=index_name)
            except elasticsearch.exceptions.NotFoundError:
                """Ignore non existent indexes."""



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0033_alter_workflowtransitionfield_options')
    ]

    operations = [
        migrations.AddField(
            model_name='workflowtransitionfield', name='default',
            field=models.TextField(
                blank=True, help_text='Optional default value for the field. '
                'Can be a template.', null=True, verbose_name='Default'
            )
        )
    ]



from django.conf import settings
from django.utils.translation import gettext_lazy as _

COMMAND_NAME_COMMON_INITIAL_SETUP = 'common_initial_setup'
COMMAND_NAME_COMMON_PERFORM_UPGRADE = 'common_perform_upgrade'
COMMAND_NAME_MIGRATE = 'migrate'

DEFAULT_COMMON_DISABLE_LOCAL_STORAGE = False
DEFAULT_COMMON_DISABLED_APPS = settings.COMMON_DISABLED_APPS
DEFAULT_COMMON_EXTRA_APPS = settings.COMMON_EXTRA_APPS
DEFAULT_COMMON_EXTRA_APPS_PRE = settings.COMMON_EXTRA_APPS_PRE
DEFAULT_COMMON_HOME_VIEW = 'common:home'
DEFAULT_COMMON_HOME_VIEW_DASHBOARD_NAME = 'user'
DEFAULT_COMMON_PROJECT_TITLE = None

DJANGO_SQLITE_BACKEND = 'django.db.backends.sqlite3'

MESSAGE_DEPRECATION_WARNING = _(
    message='This feature has been deprecated and will be removed in a future '
    'version.'
)

TIME_DELTA_UNIT_DAYS = 'days'
TIME_DELTA_UNIT_HOURS = 'hours'
TIME_DELTA_UNIT_MINUTES = 'minutes'

TIME_DELTA_UNIT_CHOICES = (
    (TIME_DELTA_UNIT_DAYS, _(message='Days')),
    (TIME_DELTA_UNIT_HOURS, _(message='Hours')),
    (TIME_DELTA_UNIT_MINUTES, _(message='Minutes'))
)

URL_BOOK = 'https://www.mayan-edms.com/book/'
URL_DOCUMENTATION = 'https://docs.mayan-edms.com'
URL_FORUM = 'https://forum.mayan-edms.com'
URL_KNOWLEDGE_BASE = 'https://forum.mayan-edms.com/t/availability-of-the-new-knowledge-base/883'
URL_MAILING_LIST = 'https://list.mayan-edms.com/subscription/form'
URL_RELEASE_NOTES = 'https://docs.mayan-edms.com/releases/index.html'
URL_SOURCE_CODE = 'https://gitlab.com/mayan-edms/mayan-edms'
URL_STORE = 'https://teespring.com/stores/mayan-edms'
URL_SUPPORT = 'https://www.mayan-edms.com/support/'



from django.utils.translation import gettext_lazy as _

from mayan.apps.platforms.platform_templates import PlatformTemplate, Variable
from mayan.settings.literals import (
    FORGE_DOCKER_BASE_IMAGE_NAME, FORGE_DOCKER_BASE_IMAGE_TAG,
    FORGE_DOCKER_COMPOSE_PROJECT_NAME, FORGE_DOCKER_IMAGE_NAME,
    FORGE_PYTHON_PACKAGES, FORGE_PYTHON_VERSION, FORGE_TRANSIFEX_VERSION,
    LINUX_PACKAGES_DEBIAN_BASE, LINUX_PACKAGES_DEBIAN_BUILD,
    LINUX_PACKAGES_DEBIAN_DOCUMENTATION, LINUX_PACKAGES_DEBIAN_FORGE,
    LINUX_PACKAGES_DEBIAN_MYSQL, LINUX_PACKAGES_DEBIAN_POSTGRESQL,
    LINUX_PACKAGES_DEBIAN_PUSH, LINUX_PACKAGES_DEBIAN_PYTHON,
    LINUX_PACKAGES_DEBIAN_TEST
)


class PlatformTemplateForgeDockerComposefile(PlatformTemplate):
    label = _(
        message='Template that generates the Mayan Forge Docker Compose file.'
    )
    name = 'forge_docker_compose'
    template_name = 'platform/forge/docker-compose.yml.tmpl'

    def __init__(self):
        with open(file='docker/rootfs/version', mode='r') as file_object:
            version_string = file_object.readline()
            version_string_clean = version_string.strip()

        self.variables = (
            Variable(
                name='FORGE_DOCKER_COMPOSE_PROJECT_NAME',
                default=FORGE_DOCKER_COMPOSE_PROJECT_NAME,
                environment_name='MAYAN_FORGE_DOCKER_COMPOSE_PROJECT_NAME'
            ),
            Variable(
                name='FORGE_DOCKER_COMPOSE_PROJECT_VERSION',
                default=version_string_clean,
                environment_name='MAYAN_FORGE_DOCKER_COMPOSE_PROJECT_VERSION'
            ),
            Variable(
                name='FORGE_DOCKER_IMAGE_NAME',
                default=FORGE_DOCKER_IMAGE_NAME,
                environment_name='MAYAN_FORGE_DOCKER_IMAGE_NAME'
            ),
            Variable(
                name='FORGE_DOCKER_IMAGE_TAG',
                default=version_string_clean,
                environment_name='MAYAN_FORGE_DOCKER_IMAGE_TAG'
            )
        )


class PlatformTemplateForgeDockerfile(PlatformTemplate):
    label = _(
        message='Template that generates a Mayan Forge Dockerfile file.'
    )
    name = 'forge_dockerfile'
    template_name = 'platform/forge/dockerfile.tmpl'

    def __init__(self):
        with open(file='docker/rootfs/version', mode='r') as file_object:
            version_string = file_object.readline()
            version_string_clean = version_string.strip()

        self.variables = (
            Variable(
                name='FORGE_DOCKER_BASE_IMAGE_NAME',
                default=FORGE_DOCKER_BASE_IMAGE_NAME,
                environment_name='MAYAN_FORGE_DOCKER_BASE_IMAGE_NAME'
            ),
            Variable(
                name='FORGE_DOCKER_BASE_IMAGE_TAG',
                default=FORGE_DOCKER_BASE_IMAGE_TAG,
                environment_name='MAYAN_FORGE_DOCKER_BASE_IMAGE_TAG'
            ),
            Variable(
                name='FORGE_PYTHON_PACKAGES',
                default=FORGE_PYTHON_PACKAGES,
                environment_name='MAYAN_FORGE_PYTHON_PACKAGES'
            ),
            Variable(
                name='FORGE_VIRTUALENV_SUFFIX',
                default=version_string_clean,
                environment_name='MAYAN_FORGE_VIRTUALENV_SUFFIX'
            ),
            Variable(
                name='FORGE_PYTHON_VERSION',
                default=FORGE_PYTHON_VERSION,
                environment_name='MAYAN_FORGE_PYTHON_VERSION'
            ),
            Variable(
                name='FORGE_TRANSIFEX_VERSION',
                default=FORGE_TRANSIFEX_VERSION,
                environment_name='MAYAN_FORGE_TRANSIFEX_VERSION'
            ),
            Variable(
                name='FORGE_VIRTUALENV_SUFFIX',
                default=version_string_clean,
                environment_name='MAYAN_FORGE_VIRTUALENV_SUFFIX'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_BASE',
                default=LINUX_PACKAGES_DEBIAN_BASE,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_BASE'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_BUILD',
                default=LINUX_PACKAGES_DEBIAN_BUILD,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_BUILD'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_DOCUMENTATION',
                default=LINUX_PACKAGES_DEBIAN_DOCUMENTATION,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_DOCUMENTATION'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_FORGE',
                default=LINUX_PACKAGES_DEBIAN_FORGE,
                environment_name='MAYAN_LINUX_PACKAGES_DEBIAN_FORGE'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_MYSQL',
                default=LINUX_PACKAGES_DEBIAN_MYSQL,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_MYSQL'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_POSTGRESQL',
                default=LINUX_PACKAGES_DEBIAN_POSTGRESQL,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_POSTGRESQL'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_PUSH',
                default=LINUX_PACKAGES_DEBIAN_PUSH,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_PUSH'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_PYTHON',
                default=LINUX_PACKAGES_DEBIAN_PYTHON,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_PYTHON'
            ),
            Variable(
                name='LINUX_PACKAGES_DEBIAN_TEST',
                default=LINUX_PACKAGES_DEBIAN_TEST,
                environment_name='MAYAN_FORGE_LINUX_PACKAGES_DEBIAN_TEST'
            )
        )



from django.apps import apps
from django.core.management.commands.makemigrations import Command as DjangoCommand

from ...literals import DJANGO_IGNORE_APP_MIGRATIONS


class Command(DjangoCommand):
    def handle(self, *app_labels, **options):
        if not app_labels:
            app_labels = {
                app_config.label for app_config in apps.get_app_configs()
            }
            app_labels -= DJANGO_IGNORE_APP_MIGRATIONS

        return super().handle(*app_labels, **options)



from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.search import search_model_document

# Document

search_model_document.add_model_field(
    field='workflows__log_entries__comment',
    label=_(message='Document workflow transition comment')
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.dynamic_search.search_models import SearchModel
from mayan.apps.views.literals import LIST_MODE_CHOICE_ITEM

from .permissions import (
    permission_document_file_view, permission_document_type_view,
    permission_document_version_view, permission_document_view
)

# Document

search_model_document = SearchModel(
    app_label='documents', default=True, label=_(message='Document'),
    list_mode=LIST_MODE_CHOICE_ITEM, manager_name='valid',
    model_name='DocumentSearchResult', permission=permission_document_view,
    serializer_path='mayan.apps.documents.serializers.document_serializers.DocumentSerializer'
)
search_model_document.add_proxy_model(
    app_label='documents', model_name='Document'
)
search_model_document.add_proxy_model(
    app_label='documents', model_name='RecentlyAccessedDocumentProxy'
)
search_model_document.add_proxy_model(
    app_label='documents', model_name='RecentlyCreatedDocument'
)

search_model_document.add_model_field(
    field='document_type__id',
    help_text=_(message='The database ID of the document type.'),
    label=_(message='Document type ID')
)
search_model_document.add_model_field(
    field='document_type__label', label=_(message='Document type label')
)
search_model_document.add_model_field(field='datetime_created')
search_model_document.add_model_field(field='label')
search_model_document.add_model_field(field='description')
search_model_document.add_model_field(field='language')
search_model_document.add_model_field(field='uuid')
search_model_document.add_model_field(
    field='files__checksum', label=('Document file checksum')
)
search_model_document.add_model_field(
    field='files__filename', label=('Document file filename')
)
search_model_document.add_model_field(
    field='files__mimetype', label=('Document file MIME type')
)

# Document file

search_model_document_file = SearchModel(
    app_label='documents', label=_(message='Document file'),
    list_mode=LIST_MODE_CHOICE_ITEM, manager_name='valid',
    model_name='DocumentFileSearchResult',
    permission=permission_document_file_view,
    serializer_path='mayan.apps.documents.serializers.document_file_serializers.DocumentFileSerializer'
)
search_model_document_file.add_proxy_model(
    app_label='documents', model_name='DocumentFile'
)

search_model_document_file.add_model_field(
    field='document__document_type__label',
    label=_(message='Document type label')
)
search_model_document_file.add_model_field(
    field='document__description', label=_(message='Document description')
)
search_model_document_file.add_model_field(
    field='document__id',
    help_text=_(message='The database ID of the document.'),
    label=_(message='Document ID')
)
search_model_document_file.add_model_field(
    field='document__label', label=_(message='Document label')
)
search_model_document_file.add_model_field(
    field='document__uuid', label=_(message='Document UUID')
)
search_model_document_file.add_model_field(field='checksum')
search_model_document_file.add_model_field(field='comment')
search_model_document_file.add_model_field(field='filename')
search_model_document_file.add_model_field(field='mimetype')
search_model_document_file.add_model_field(field='size')

# Document file page

search_model_document_file_page = SearchModel(
    app_label='documents', label=_(message='Document file page'),
    list_mode=LIST_MODE_CHOICE_ITEM, manager_name='valid',
    model_name='DocumentFilePageSearchResult',
    permission=permission_document_file_view,
    serializer_path='mayan.apps.documents.serializers.document_file_serializers.DocumentFilePageSerializer'
)
search_model_document_file_page.add_proxy_model(
    app_label='documents', model_name='DocumentFilePage'
)

search_model_document_file_page.add_model_field(
    field='document_file__document__document_type__label',
    label=_(message='Document type label')
)
search_model_document_file_page.add_model_field(
    field='document_file__document__label', label=_(message='Document label')
)
search_model_document_file_page.add_model_field(
    field='document_file__checksum', label=_(message='Document file checksum')
)
search_model_document_file_page.add_model_field(
    field='document_file__document__uuid', label=_(message='Document UUID')
)

# Document type

search_model_document_type = SearchModel(
    app_label='documents', list_mode=LIST_MODE_CHOICE_ITEM,
    model_name='DocumentType', permission=permission_document_type_view,
    serializer_path='mayan.apps.documents.serializers.document_type_serializers.DocumentTypeSerializer'
)
search_model_document_type.add_model_field(field='id')
search_model_document_type.add_model_field(field='label')

# Document version

search_model_document_version = SearchModel(
    app_label='documents', label=_(message='Document version'),
    list_mode=LIST_MODE_CHOICE_ITEM, manager_name='valid',
    model_name='DocumentVersionSearchResult',
    permission=permission_document_version_view,
    serializer_path='mayan.apps.documents.serializers.document_version_serializers.DocumentVersionSerializer'
)
search_model_document_version.add_proxy_model(
    app_label='documents', model_name='DocumentVersion'
)

search_model_document_version.add_model_field(field='comment')
search_model_document_version.add_model_field(
    field='document__document_type__label',
    label=_(message='Document type label')
)
search_model_document_version.add_model_field(
    field='document__id',
    help_text=_(message='The database ID of the document.'),
    label=_(message='Document ID')
)
search_model_document_version.add_model_field(
    field='document__label', label=_(message='Document label')
)
search_model_document_version.add_model_field(
    field='document__description', label=_(message='Document description')
)
search_model_document_version.add_model_field(
    field='document__uuid', label=_(message='Document UUID')
)

# Document version page

search_model_document_version_page = SearchModel(
    app_label='documents', label=_(message='Document version page'),
    list_mode=LIST_MODE_CHOICE_ITEM, manager_name='valid',
    model_name='DocumentVersionPageSearchResult',
    permission=permission_document_version_view,
    serializer_path='mayan.apps.documents.serializers.document_version_serializers.DocumentVersionPageSerializer'
)
search_model_document_version_page.add_proxy_model(
    app_label='documents', model_name='DocumentVersionPage'
)

search_model_document_version_page.add_model_field(
    field='document_version__document__document_type__label',
    label=_(message='Document type label')
)
search_model_document_version_page.add_model_field(
    field='document_version__document__label',
    label=_(message='Document label')
)
search_model_document_version_page.add_model_field(
    field='document_version__document__uuid',
    label=_(message='Document UUID')
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.authentication.link_conditions import (
    condition_user_is_authenticated
)
from mayan.apps.authentication.utils import get_context_user
from mayan.apps.navigation.links import Link

from ..icons import (
    icon_document_favorite_add_multiple, icon_document_favorite_add_single,
    icon_document_favorite_list, icon_document_favorite_remove_multiple,
    icon_document_favorite_remove_single
)
from ..permissions import permission_document_view


def condition_is_in_favorites(context, resolved_object):
    if condition_user_is_authenticated(
        context=context, resolved_object=resolved_object
    ):
        user = get_context_user(context=context)
        return resolved_object.favorites.filter(user=user).exists()


def condition_not_is_in_favorites(context, resolved_object):
    if condition_user_is_authenticated(
        context=context, resolved_object=resolved_object
    ):
        user = get_context_user(context=context)
        return not resolved_object.favorites.filter(user=user).exists()


link_document_favorites_add_multiple = Link(
    text=_(message='Add to favorites'),
    icon=icon_document_favorite_add_multiple,
    view='documents:document_favorite_add_multiple'
)
link_document_favorites_add_single = Link(
    condition=condition_not_is_in_favorites,
    args='resolved_object.id', icon=icon_document_favorite_add_single,
    permission=permission_document_view, text=_(message='Add to favorites'),
    view='documents:document_favorite_add'
)
link_document_favorites_list = Link(
    condition=condition_user_is_authenticated,
    icon=icon_document_favorite_list, text=_(message='Favorites'),
    view='documents:document_favorite_list'
)
link_document_favorites_remove_multiple = Link(
    text=_(message='Remove from favorites'),
    icon=icon_document_favorite_remove_multiple,
    view='documents:document_favorite_remove_multiple'
)
link_document_favorites_remove_single = Link(
    condition=condition_is_in_favorites,
    args='resolved_object.id', icon=icon_document_favorite_remove_single,
    permission=permission_document_view, text=_(
        message='Remove from favorites'
    ), view='documents:document_favorite_remove'
)



from django.urls import re_path

from .api_views import APIDocumentVersionExportView
from .views import DocumentVersionExportView

urlpatterns = [
    re_path(
        route=r'^documents/versions/(?P<document_version_id>\d+)/export/$',
        name='document_version_export',
        view=DocumentVersionExportView.as_view()
    )
]

api_urls = [
    re_path(
        route=r'^documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/export/$',
        view=APIDocumentVersionExportView.as_view(),
        name='documentversion-export'
    )
]



from mayan.apps.dependencies.environments import environment_build

from .classes import PythonDependency

PythonDependency(
    module=__name__, name='node-semver', version_string='==0.9.0'
)
PythonDependency(
    environment=environment_build, module=__name__, name='packaging',
    version_string='==25.0'
)



from django.utils.translation import gettext_lazy as _

DEFAULT_APT_INSTALLS = ''
DEFAULT_DOCKER_SCRIPT_POST_SETUP = ''
DEFAULT_DOCKER_SCRIPT_PRE_SETUP = ''
DEFAULT_DOCKER_WAIT = ''
DEFAULT_PIP_INSTALLS = ''
DEFAULT_SKIP_CHOWN_ON_STARTUP = False

worker_setting_map = (
    (
        'concurrency', {
            'default_name': 'CONCURRENCY',
            'help_text': _(
                message='Allows setting the worker\'s Celery `concurrency` '
                'value.'
            )
        }
    ),
    (
        'maximum_memory_per_child', {
            'default_name': 'MAX_MEMORY_PER_CHILD',
            'help_text': _(
                message='Allows setting the worker\'s Celery '
                '`max-memory-per-child` value.'
            )
        }
    ),
    (
        'maximum_tasks_per_child', {
            'default_name': 'MAX_TASKS_PER_CHILD',
            'help_text': _(
                message='Allows setting the worker\'s Celery '
                '`max-tasks-per-child` value.'
            )
        }
    ),
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.links import Link

from .icons import (
    icon_document_type_ocr_settings, icon_document_type_ocr_submit,
    icon_document_version_ocr_content_delete_multiple,
    icon_document_version_ocr_content_delete_single,
    icon_document_version_ocr_content_detail,
    icon_document_version_ocr_content_download,
    icon_document_version_ocr_submit_multiple,
    icon_document_version_ocr_submit_single,
    icon_document_version_page_ocr_content_detail,
    icon_document_version_page_ocr_content_edit
)
from .permissions import (
    permission_document_type_ocr_setup, permission_document_version_ocr,
    permission_document_version_ocr_content_edit,
    permission_document_version_ocr_content_view
)

# Document type

link_document_type_ocr_settings = Link(
    args='resolved_object.id', icon=icon_document_type_ocr_settings,
    permission=permission_document_type_ocr_setup,
    text=_(message='OCR setup'), view='ocr:document_type_ocr_settings'
)
link_document_type_submit = Link(
    icon=icon_document_type_ocr_submit,
    permission=permission_document_version_ocr,
    text=_(message='OCR documents per type'), view='ocr:document_type_submit'
)

# Document version

link_document_version_ocr_content_delete_multiple = Link(
    icon=icon_document_version_ocr_content_delete_multiple,
    text=_(message='Delete OCR content'),
    view='ocr:document_version_ocr_content_multiple_delete'
)
link_document_version_ocr_content_delete_single = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_content_delete_single,
    permission=permission_document_version_ocr,
    text=_(message='Delete OCR content'),
    view='ocr:document_version_ocr_content_single_delete'
)
link_document_version_ocr_content_detail = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_content_detail,
    permission=permission_document_version_ocr_content_view,
    text=_(message='OCR'), view='ocr:document_version_ocr_content_view'
)
link_document_version_ocr_content_download = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_content_download,
    permission=permission_document_version_ocr_content_view,
    text=_(message='Download OCR text'),
    view='ocr:document_version_ocr_content_download'
)
link_document_version_ocr_submit_multiple = Link(
    icon=icon_document_version_ocr_submit_multiple,
    text=_(message='Submit for OCR'), view='ocr:document_version_ocr_multiple_submit'
)
link_document_version_ocr_submit_single = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_submit_single,
    permission=permission_document_version_ocr, text=_(message='Submit for OCR'),
    view='ocr:document_version_ocr_single_submit'
)

# Document version page

link_document_version_page_ocr_content_detail_view = Link(
    args='resolved_object.id',
    icon=icon_document_version_page_ocr_content_detail,
    permission=permission_document_version_ocr_content_view,
    text=_(message='OCR'), view='ocr:document_version_page_ocr_content_detail_view'
)
link_document_version_page_ocr_content_edit_view = Link(
    args='resolved_object.id',
    icon=icon_document_version_page_ocr_content_edit,
    permission=permission_document_version_ocr_content_edit,
    text=_(message='Edit OCR'),
    view='ocr:document_version_page_ocr_content_edit_view'
)



from django.conf import settings
from django.http import Http404, HttpResponseRedirect
from django.template import TemplateSyntaxError
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from mayan.apps.views.generics import FormView
from mayan.apps.views.http import URL
from mayan.apps.views.view_mixins import (
    ContentTypeViewMixin, ExternalObjectViewMixin
)

from .classes import ModelTemplating
from .forms import TemplateSandboxForm
from .icons import icon_template_sandbox
from .permissions import permission_template_sandbox


class ObjectTemplateSandboxView(
    ContentTypeViewMixin, ExternalObjectViewMixin, FormView
):
    content_type_url_kw_args = {
        'app_label': 'app_label',
        'model_name': 'model_name'
    }
    external_object_permission = permission_template_sandbox
    external_object_pk_url_kwarg = 'object_id'
    form_class = TemplateSandboxForm
    view_icon = icon_template_sandbox

    def form_valid(self, form):
        content_type = self.get_content_type()
        path = reverse(
            kwargs={
                'app_label': content_type.app_label,
                'model_name': content_type.model,
                'object_id': self.external_object.pk
            }, viewname='templating:object_template_sandbox'
        )
        url = URL(
            path=path, query={
                'template': form.cleaned_data['template']
            }
        )

        return HttpResponseRedirect(
            redirect_to=url.to_string()
        )

    def get_external_object_queryset(self):
        # Here we get a queryset the object model for which an ACL will be
        # created.
        return self.get_content_type().get_all_objects_for_this_type()

    def get_extra_context(self):
        return {
            'object': self.external_object,
            'title': _(
                message='Template sandbox for: %s'
            ) % self.external_object
        }

    def get_form_extra_kwargs(self):
        model_templating = self.get_model_templating()

        return {
            'model': model_templating.model,
            'model_variable': model_templating.variable_name
        }

    def get_initial(self):
        if settings.DEBUG:
            exception_list = (TemplateSyntaxError,)
        else:
            exception_list = (Exception, TemplateSyntaxError,)

        template_string = self.request.GET.get('template', '')

        try:
            result = ModelTemplating.do_render(
                obj=self.external_object, template_string=template_string
            )
        except exception_list as exception:
            result = ''
            error_message = _(
                message='Template error; %(exception)s'
            ) % {'exception': exception}

            result = error_message
        except KeyError as exception:
            raise Http404(exception)

        return {'result': result, 'template': template_string}

    def get_model_templating(self):
        model = self.external_object._meta.model
        try:
            model_templating = ModelTemplating.get_for_model(model=model)
        except KeyError as exception:
            raise Http404(exception)

        return model_templating



import logging

from django.shortcuts import reverse
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.models.document_models import Document
from mayan.apps.views.generics import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectDetailView,
    SingleObjectEditView, SingleObjectListView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from .forms import SignatureCaptureForm, SignatureCaptureDetailForm
from .icons import (
    icon_signature_captures, icon_signature_capture_create,
    icon_signature_capture_single_delete, icon_signature_capture_edit,
    icon_signature_capture_list
)
from .links import link_signature_capture_create
from .models import SignatureCapture
from .permissions import (
    permission_signature_capture_create, permission_signature_capture_delete,
    permission_signature_capture_edit, permission_signature_capture_view
)

logger = logging.getLogger(name=__name__)


class SignatureCaptureCreateView(
    ExternalObjectViewMixin, SingleObjectCreateView
):
    external_object_permission = permission_signature_capture_create
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    form_class = SignatureCaptureForm
    view_icon = icon_signature_capture_create

    def get_extra_context(self):
        return {
            'object': self.external_object,
            'title': _(
                message='Create signature capture for document: %s'
            ) % self.external_object
        }

    def get_instance_extra_data(self):
        return {
            'document': self.external_object,
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(
            kwargs={'document_id': self.external_object.pk},
            viewname='signature_captures:signature_capture_list'
        )


class SignatureCaptureDeleteView(SingleObjectDeleteView):
    object_permission = permission_signature_capture_delete
    pk_url_kwarg = 'signature_capture_id'
    source_queryset = SignatureCapture.valid.all()
    view_icon = icon_signature_capture_single_delete

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}

    def get_extra_context(self):
        return {
            'document': self.object.document,
            'navigation_object_list': ('object', 'document'),
            'object': self.object,
            'title': _(message='Delete signature capture: %s') % self.object
        }

    def get_post_action_redirect(self):
        return reverse_lazy(
            viewname='signature_captures:signature_capture_list', kwargs={
                'document_id': self.object.document.pk
            }
        )


class SignatureCaptureDetailView(SingleObjectDetailView):
    form_class = SignatureCaptureDetailForm
    object_permission = permission_signature_capture_view
    pk_url_kwarg = 'signature_capture_id'
    source_queryset = SignatureCapture.valid.all()
    view_icon = icon_signature_captures

    def get_extra_context(self):
        return {
            'navigation_object_list': ('object', 'object.document'),
            'object': self.object,
            'title': _(message='Details of: %s') % self.object
        }


class SignatureCaptureEditView(SingleObjectEditView):
    form_class = SignatureCaptureForm
    object_permission = permission_signature_capture_edit
    pk_url_kwarg = 'signature_capture_id'
    source_queryset = SignatureCapture.valid.all()
    view_icon = icon_signature_capture_edit

    def get_extra_context(self):
        return {
            'document': self.object.document,
            'navigation_object_list': ('object', 'document'),
            'object': self.object,
            'title': _(message='Edit document signature capture: %s') % self.object
        }

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class SignatureCaptureListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    object_permission = permission_signature_capture_view
    view_icon = icon_signature_capture_list

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_signature_captures,
            'no_results_main_link': link_signature_capture_create.resolve(
                context=RequestContext(
                    dict_={'object': self.external_object},
                    request=self.request
                )
            ),
            'no_results_text': _(
                message='Signature captures are electronic versions of a '
                'persons\'s handwritten signature.'
            ),
            'no_results_title': _(message='Document has no signature captures'),
            'object': self.external_object,
            'title': _(
                message='Signature captures for document: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.signature_captures.all()



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import EventManagerSave

from .events import event_announcement_created, event_announcement_edited
from .managers import AnnouncementManager


class Announcement(ExtraDataModelMixin, models.Model):
    """
    Model to store an information announcement that will be displayed at the
    login screen. Announcements can have an activation and deactivation date.
    """
    _ordering_fields = ('enabled', 'end_datetime', 'label', 'start_datetime')

    label = models.CharField(
        max_length=32, help_text=_(
            message='Short description of this announcement.'
        ), verbose_name=_(message='Label')
    )
    text = models.TextField(
        help_text=_(message='The actual text to be displayed.'),
        verbose_name=_(message='Text')
    )
    enabled = models.BooleanField(
        default=True, verbose_name=_(message='Enabled')
    )
    start_datetime = models.DateTimeField(
        blank=True, help_text=_(
            message='Date and time after which this announcement will be displayed.'
        ), null=True, verbose_name=_(message='Start date time')
    )
    end_datetime = models.DateTimeField(
        blank=True, help_text=_(
            message='Date and time until when this announcement is to be displayed.'
        ), null=True, verbose_name=_(message='End date time')
    )

    objects = AnnouncementManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _(message='Announcement')
        verbose_name_plural = _(message='Announcements')

    def __str__(self):
        return self.label

    def get_absolute_url(self):
        return reverse(viewname='announcements:announcement_list')

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_announcement_created,
            'target': 'self'
        },
        edited={
            'event': event_announcement_edited,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)



from django.utils.translation import gettext_lazy as _

from mayan.apps.document_states.classes import WorkflowAction

from .models import Tag
from .permissions import permission_tag_attach, permission_tag_remove

__all__ = ('AttachTagAction', 'RemoveTagAction')


class AttachTagAction(WorkflowAction):
    form_field_widgets = {
        'tags': {
            'class': 'mayan.apps.tags.widgets.TagFormWidget', 'kwargs': {
                'attrs': {'class': 'select2-tags'}
            }
        }
    }
    label = _(message='Attach tag')
    form_media = {
        'js': ('tags/js/tags_form.js',)
    }
    permission = permission_tag_attach

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        fields.update(
            {
                'tags': {
                    'class': 'mayan.apps.forms.form_fields.ModelFormFieldFilteredModelMultipleChoice',
                    'help_text': _(
                        message='Tags to attach to the document.'
                    ),
                    'kwargs': {
                        'source_model': Tag,
                        'permission': cls.permission
                    },
                    'label': _(message='Tags'),
                    'required': True
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _(message='Tags'), {
                    'fields': ('tags',)
                },
            ),
        )
        return fieldsets

    def execute(self, context):
        for tag in self.get_tags():
            tag._attach_to(
                document=context['workflow_instance'].document
            )

    def get_tags(self):
        return Tag.objects.filter(
            pk__in=self.kwargs.get(
                'tags', ()
            )
        )


class RemoveTagAction(AttachTagAction):
    label = _(message='Remove tag')
    permission = permission_tag_remove

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        fields['tags']['help_text'] = _(message='Tags to remove from the document.')

        return fields

    def execute(self, context):
        for tag in self.get_tags():
            tag._remove_from(
                document=context['workflow_instance'].document
            )



from django.conf import settings
from django.template import Library

from ..decorators import templating_dangerous_tag

register = Library()


if settings.TESTING:
    from ..tests.literals import TEST_TEMPLATE_TAG_RESULT
    # Hidden import.
    # Required to allow production package to work when tests are
    # removed.

    @register.filter(name='dangerous_filter')
    @templating_dangerous_tag()
    def templating_test_filter_dangerous(value):
        """
        Test docstring dangerous filter
        """
        return TEST_TEMPLATE_TAG_RESULT

    @register.simple_tag
    def templating_test_tag():
        """
        Test docstring
        """
        return TEST_TEMPLATE_TAG_RESULT

    @register.simple_tag(name='dangerous_tag')
    @templating_dangerous_tag()
    def templating_test_tag_dangerous():
        """
        Test docstring dangerous tag
        """
        return TEST_TEMPLATE_TAG_RESULT



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.menus import Menu

from .icons import icon_menu_documents

menu_documents = Menu(
    icon=icon_menu_documents, label=_(message='Documents'), name='documents'
)



from django.apps import apps
from django.utils.translation import gettext_lazy as _

from .permissions import permission_credential_use


class BackendMixinCredentials:
    @classmethod
    def get_form_fields(cls):
        StoredCredential = apps.get_model(
            app_label='credentials', model_name='StoredCredential'
        )

        fields = super().get_form_fields()

        fields.update(
            {
                'stored_credential_id': {
                    'class': 'mayan.apps.forms.form_fields.FormFieldFilteredModelChoice',
                    'help_text': _(
                        message='The credential entry to use for '
                        'authentication.'
                    ),
                    'kwargs': {
                        'source_model': StoredCredential,
                        'permission': permission_credential_use
                    },
                    'label': _(message='Credential'),
                    'required': True
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _(message='Authentication'), {
                    'fields': (
                        'stored_credential_id',
                    )
                },
            ),
        )

        return fieldsets

    def get_credential(self, action_object=None, user=None):
        StoredCredential = apps.get_model(
            app_label='credentials', model_name='StoredCredential'
        )

        stored_credential_id = self.kwargs.get('stored_credential_id')

        if stored_credential_id:
            stored_credential = StoredCredential.objects.get(
                pk=stored_credential_id
            )

            backend_instance = stored_credential.get_backend_instance()

            return backend_instance.get_credential(
                action_object=action_object, user=user
            )


class BackendMixinCredentialsOptional(BackendMixinCredentials):
    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        fields['stored_credential_id']['required'] = False
        fields['stored_credential_id']['help_text'] = _(
            message='Optional credential entry to use for authentication.'
        )

        return fields



import yaml

from django.utils.html import mark_safe


class Dumper(yaml.Dumper):
    def increase_indent(self, flow=False, *args, **kwargs):
        return super().increase_indent(flow=flow, indentless=False)


def yaml_dump(data, indent):
    result = yaml.dump(
        Dumper=Dumper, data=data, width=1000
    )

    result = result.replace('\'\'\'', '\'')

    output = []

    for line in result.split('\n'):
        if line:
            output.append(
                '{}{}'.format(
                    ' ' * indent, line
                )
            )

    return mark_safe(
        '\n'.join(output)
    )


class EnvironmentFileLoader:
    FILE_NAME_CONFIG_ENV = 'config.env'
    FILE_NAME_CONFIG_LOCALE_ENV = 'config-local.env'

    def __init__(self, filename=FILE_NAME_CONFIG_ENV, skip_local_config=False):
        self.filename = filename
        self.skip_local_config = skip_local_config

    def _do_file_object_content_load(self, file_object):
        result = {}

        for line in file_object:
            key, value = self._do_line_process(line=line)

            if key is not None:
                result[key] = value

        return result

    def _do_line_process(self, line):
        """
        >>> EnvironmentFileLoader()._do_line_process(line='A=1')
        ('A', '1')
        >>> EnvironmentFileLoader()._do_line_process(line='A="1"')
        ('A', '"1"')
        >>> EnvironmentFileLoader()._do_line_process(line='A=a==1')
        ('A', 'a==1')
        """
        key = None
        value = None

        line_clean = line.strip()

        if line_clean and not line_clean.startswith('#'):
            key, value = line_clean.split('=', 1)

        return key, value

    def do_content_load(self):
        with open(file=self.filename) as file_object:
            config_content = self._do_file_object_content_load(
                file_object=file_object
            )

        if self.filename != EnvironmentFileLoader.FILE_NAME_CONFIG_LOCALE_ENV and not self.skip_local_config:
            try:
                with open(file=EnvironmentFileLoader.FILE_NAME_CONFIG_LOCALE_ENV) as file_object:
                    config_local_content = self._do_file_object_content_load(
                        file_object=file_object
                    )
            except FileNotFoundError:
                """
                Non fatal. Just means this deployment does not overrides the
                default `config.env` file values.
                """
            else:
                config_content.update(config_local_content)

        return config_content


def load_env_file(*args, **kwargs):
    instance = EnvironmentFileLoader(*args, **kwargs)
    result = instance.do_content_load()
    return result



import email
from io import BytesIO
import tarfile
import zipfile

import extract_msg
from pypdf import PdfReader

try:
    import zlib  # NOQA
    COMPRESSION = zipfile.ZIP_DEFLATED
except ImportError:
    COMPRESSION = zipfile.ZIP_STORED

from django.core.files import File
from django.core.files.uploadedfile import SimpleUploadedFile
from django.utils.encoding import force_bytes, force_str

from mayan.apps.mime_types.classes import MIMETypeBackend

from .exceptions import NoMIMETypeMatch
from .literals import MIME_TYPE_EML, MSG_MIME_TYPES


class Archive:
    _registry = {}

    @classmethod
    def register(cls, mime_types, archive_classes):
        for mime_type in mime_types:
            for archive_class in archive_classes:
                cls._registry.setdefault(
                    mime_type, []
                ).append(archive_class)

    @classmethod
    def open(cls, file_object):
        mime_type = MIMETypeBackend.get_backend_instance().get_mime_type(
            file_object=file_object, mime_type_only=True
        )[0]

        try:
            archives_classes = cls._registry[mime_type]
        except KeyError:
            raise NoMIMETypeMatch
        else:
            for archive_class in archives_classes:
                instance = archive_class()
                instance._open(file_object=file_object)
                return instance

    def _open(self, file_object):
        raise NotImplementedError

    def add_file(self, file_object, filename):
        """
        Add a file as a member of an archive
        """
        raise NotImplementedError

    def close(self):
        self._archive.close()

    def create(self):
        """
        Create an empty archive
        """
        raise NotImplementedError

    def get_members(self):
        return (
            SimpleUploadedFile(
                content=self.member_contents(filename=filename),
                name=filename
            ) for filename in self.members()
        )

    def member_contents(self, filename):
        """
        Return the content of a member
        """
        raise NotImplementedError

    def members(self):
        """
        Return a list of all the elements inside the archive
        """
        raise NotImplementedError

    def open_member(self, filename):
        """
        Return a file-like object to a member of the archive
        """
        raise NotImplementedError


class EMLArchive(Archive):
    def _get_parts(self, message):
        counter = 1

        if message.is_multipart():
            for part in message.iter_parts():
                yield from self._get_parts(message=part)
        else:
            if message.is_attachment() or message.get_content_disposition() == 'inline':
                content = message.get_content()
                if len(content) != 0:
                    detected_filename = message.get_filename()
                    if detected_filename:
                        label = detected_filename
                    else:
                        label = 'attachment-{}'.format(counter)
                        counter += 1

                    yield {'label': label, 'message': message}
            else:
                # If it is not an attachment then it should be a body message
                # part.
                yield {'label': 'body', 'message': message}

    def _open(self, file_object):
        self._archive = email.message_from_binary_file(
            fp=file_object, policy=email.policy.default
        )

    def get_parts(self):
        yield from self._get_parts(message=self._archive)

    def member_contents(self, filename):
        for part in self.get_parts():
            if part['label'] == filename:
                return force_bytes(
                    s=part['message'].get_content()
                )

    def members(self):
        result = []
        for part in self.get_parts():
            result.append(
                part['label']
            )

        return result

    def open_member(self, filename):
        for part in self.get_parts():
            if part['label'] == filename:
                return File(
                    file=BytesIO(
                        initial_bytes=force_bytes(
                            s=part['message'].get_content()
                        )
                    ), name=filename
                )


class MsgArchive(Archive):
    def _open(self, file_object):
        self._archive = extract_msg.Message(path=file_object)

    def member_contents(self, filename):
        if filename == 'message.txt':
            return force_bytes(s=self._archive.body)

        for member in self._archive.attachments:
            if member.longFilename == filename:
                return force_bytes(s=member.data)

    def members(self):
        results = []
        for attachments in self._archive.attachments:
            results.append(attachments.longFilename)

        if self._archive.body:
            results.append('message.txt')

        return results

    def open_member(self, filename):
        if filename == 'message.txt':
            return File(
                file=BytesIO(
                    initial_bytes=force_bytes(s=self._archive.body)
                ), name=filename
            )

        for member in self._archive.attachments:
            if member.longFilename == filename:
                return File(
                    file=BytesIO(
                        initial_bytes=force_bytes(s=member.data)
                    ), name=filename
                )


class PDFArchive(Archive):
    def _get_member_and_content_list(self):
        for name, content_list in self._archive.attachments.items():
            for index, content in enumerate(iterable=content_list):
                member_filename = '{index}-{name}'.format(
                    index=index, name=name
                )
                yield member_filename, content

    def _open(self, file_object):
        self._archive = PdfReader(stream=file_object)

    def member_contents(self, filename):
        for member_filename, content in self._get_member_and_content_list():
            if filename == member_filename:
                yield from content

    def members(self):
        result = [
            member_filename for member_filename, content in self._get_member_and_content_list()
        ]

        return result

    def open_member(self, filename):
        for member_filename, content in self._get_member_and_content_list():
            if filename == member_filename:
                initial_bytes = force_bytes(s=content)
                buffer = BytesIO(initial_bytes=initial_bytes)
                return File(file=buffer, name=filename)


class TarArchive(Archive):
    def _open(self, file_object):
        self._archive = tarfile.open(fileobj=file_object)

    def add_file(self, file_object, filename):
        self._archive.addfile(
            tarinfo=self._archive.gettarinfo(
                fileobj=file_object, arcname=filename
            ), fileobj=file_object
        )

    def create(self):
        self.string_buffer = BytesIO()
        # Mode cannot be a binary mode.
        self._archive = tarfile.TarFile(fileobj=self.string_buffer, mode='w')

    def member_contents(self, filename):
        return self._archive.extractfile(filename).read()

    def members(self):
        return self._archive.getnames()

    def open_member(self, filename):
        return self._archive.extractfile(filename)


class ZipArchive(Archive):
    def _open(self, file_object):
        self._archive = zipfile.ZipFile(file=file_object)

    def add_file(self, file_object, filename):
        self._archive.writestr(
            zinfo_or_arcname=filename, data=file_object.read(),
            compress_type=COMPRESSION
        )

    def create(self):
        self.string_buffer = BytesIO()
        # Mode cannot be a binary mode.
        self._archive = zipfile.ZipFile(file=self.string_buffer, mode='w')

    def member_contents(self, filename):
        return self._archive.read(name=filename)

    def members(self):
        results = []

        for filename in self._archive.namelist():
            # Zip files only support UTF-8 and CP437 encodings.
            # Attempt to decode CP437 to be able to check if it ends
            # with a slash.
            # Future improvement that violates the Zip format:
            # Add chardet.detect to detect the most likely encoding
            # if other than CP437.
            try:
                filename = filename.decode('CP437')
                is_unicode = False
            except AttributeError:
                filename = force_str(s=filename)
                is_unicode = True
            except UnicodeEncodeError:
                is_unicode = True

            if not filename.endswith('/'):
                # Re encode in the original encoding
                if not is_unicode:
                    filename = filename.encode(
                        encoding='CP437', errors='strict'
                    )

                results.append(filename)

        return results

    def open_member(self, filename):
        return self._archive.open(name=filename)

    def write(self, filename=None):
        # fix for Linux zip files read in Windows
        for entry in self._archive.filelist:
            entry.create_system = 0

        self.string_buffer.seek(0)

        if filename:
            with open(file=filename, mode='wb') as file_object:
                file_object.write(
                    self.string_buffer.read()
                )
        else:
            return self.string_buffer

    def as_file(self, filename):
        return SimpleUploadedFile(
            name=filename, content=self.write().read()
        )


Archive.register(
    archive_classes=(EMLArchive,), mime_types=MIME_TYPE_EML
)
Archive.register(
    archive_classes=(MsgArchive,), mime_types=MSG_MIME_TYPES
)
Archive.register(
    archive_classes=(PDFArchive,), mime_types=('application/pdf',)
)
Archive.register(
    archive_classes=(TarArchive,), mime_types=('application/x-tar',)
)
Archive.register(
    archive_classes=(TarArchive,), mime_types=('application/gzip',)
)
Archive.register(
    archive_classes=(TarArchive,), mime_types=(
        'application/x-bzip', 'application/x-bzip2'
    )
)
Archive.register(
    archive_classes=(ZipArchive,), mime_types=('application/zip',)
)



from .workflow_instance_models import *  # NOQA
from .workflow_models import *  # NOQA
from .workflow_state_action_models import *  # NOQA
from .workflow_state_escalation_models import *  # NOQA
from .workflow_state_models import *  # NOQA
from .workflow_transition_field_models import *  # NOQA
from .workflow_transition_models import *  # NOQA
from .workflow_transition_trigger_models import *  # NOQA



COMMAND_NAME_PLATFORMS_TEMPLATE = 'platforms_template'

DEFAULT_PLATFORMS_CLIENT_BACKEND_ARGUMENTS = {}
DEFAULT_PLATFORMS_CLIENT_BACKEND_ENABLED = ()
DEFAULT_SETTINGS_MODULE = 'mayan.settings.production'



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Search'), name='search'
)

permission_saved_resultset_delete = namespace.add_permission(
    label=_(message='Delete resultsets'), name='saved_resultset_delete'
)
permission_saved_resultset_view = namespace.add_permission(
    label=_(message='View resultsets'), name='saved_resultset_view'
)

permission_search_tools = namespace.add_permission(
    label=_(message='Execute search tools'), name='search_tools'
)



import logging

from django.apps import apps

from .settings import setting_auto_ocr

logger = logging.getLogger(name=__name__)


def handler_initialize_new_ocr_settings(sender, instance, **kwargs):
    DocumentTypeOCRSettings = apps.get_model(
        app_label='ocr', model_name='DocumentTypeOCRSettings'
    )

    if kwargs['created']:
        DocumentTypeOCRSettings.objects.create(
            document_type=instance, auto_ocr=setting_auto_ocr.value
        )


def handler_ocr_document_version(sender, instance, **kwargs):
    logger.debug('instance pk: %s', instance.pk)
    if instance.document.document_type.ocr_settings.auto_ocr:
        instance.submit_for_ocr()



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0032_workflowtransitionfield_lookup')
    ]

    operations = [
        migrations.AlterModelOptions(
            name='workflowtransitionfield', options={
                'ordering': ('label',),
                'verbose_name': 'Workflow transition field',
                'verbose_name_plural': 'Workflow transition fields'
            }
        )
    ]



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('document_indexing', '0011_auto_20170524_0456')
    ]

    operations = [
        migrations.AlterField(
            model_name='index',
            name='slug',
            field=models.SlugField(
                help_text='This value will be used by other apps to reference '
                'this index.', max_length=128, unique=True, verbose_name='Slug'
            ),
        ),
        migrations.AlterField(
            model_name='indexinstancenode',
            name='documents',
            field=models.ManyToManyField(
                related_name='index_instance_nodes', to='documents.Document',
                verbose_name='Documents'
            ),
        ),
        migrations.AlterField(
            model_name='indexinstancenode',
            name='index_template_node',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='index_instance_nodes',
                to='document_indexing.IndexTemplateNode',
                verbose_name='Index template node'
            ),
        )
    ]



import logging

from django.apps import apps
from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.app_manager.apps import MayanAppConfig
from mayan.apps.common.menus import menu_list_facet

from .links import link_redaction_list
from .permissions import (
    permission_redaction_create, permission_redaction_delete,
    permission_redaction_edit, permission_redaction_exclude,
    permission_redaction_view
)
from .transformations import *  # NOQA

logger = logging.getLogger(name=__name__)


class RedactionsApp(MayanAppConfig):
    app_namespace = 'redactions'
    app_url = 'redactions'
    has_rest_api = False
    has_static_media = True
    has_tests = True
    name = 'mayan.apps.redactions'
    static_media_ignore_patterns = (
        'redactions/node_modules/cropperjs/src/*',
        'redactions/node_modules/cropperjs/types/index.d.ts',
        'redactions/node_modules/jquery-cropper/src/*'
    )
    verbose_name = _(message='Redactions')

    def ready(self):
        super().ready()

        DocumentFile = apps.get_model(
            app_label='documents', model_name='DocumentFile'
        )
        DocumentFilePage = apps.get_model(
            app_label='documents', model_name='DocumentFilePage'
        )
        DocumentVersion = apps.get_model(
            app_label='documents', model_name='DocumentVersion'
        )
        DocumentVersionPage = apps.get_model(
            app_label='documents', model_name='DocumentVersionPage'
        )

        ModelPermission.register(
            model=DocumentFile, permissions=(
                permission_redaction_create,
                permission_redaction_delete,
                permission_redaction_edit,
                permission_redaction_exclude,
                permission_redaction_view
            )
        )
        ModelPermission.register(
            model=DocumentVersion, permissions=(
                permission_redaction_create,
                permission_redaction_delete,
                permission_redaction_edit,
                permission_redaction_exclude,
                permission_redaction_view
            )
        )

        menu_list_facet.bind_links(
            links=(link_redaction_list,), sources=(
                DocumentFilePage, DocumentVersionPage,
            )
        )



DEFAULT_DIRECTORY_BASE = '/opt/'
DEFAULT_DIRECTORY_INSTALLATION = '/opt/mayan-edms/'
DEFAULT_DIRECTORY_MEDIA_ROOT = '/opt/mayan-edms/media/'
DEFAULT_DATABASE_NAME = 'mayan'
DEFAULT_DATABASE_PASSWORD = 'mayandbpass'
DEFAULT_DATABASE_USER = 'mayan'
DEFAULT_ELASTICSEARCH_PASSWORD = 'mayanespassword'
DEFAULT_OS_DISTRIBUTION = 'Debian 12'
DEFAULT_OS_GROUP = 'mayan'
DEFAULT_OS_USERNAME = 'mayan'
DEFAULT_RABBITMQ_CONSUMER_TIMEOUT = 1800000
DEFAULT_RABBITMQ_PASSWORD = 'mayanrabbitpass'
DEFAULT_RABBITMQ_USER = 'mayan'
DEFAULT_RABBITMQ_VHOST = 'mayan'
DEFAULT_REDIS_PASSWORD = 'mayanredispassword'
DEFAULT_SECRET_KEY = 'secret-key-missing!'
DEFAULT_USER_SETTINGS_FOLDER = 'user_settings/'
DEFAULT_USER_SETTINGS_MODULE = 'user_settings'
DJANGO_SERIES = '4.2'
DOCKER_HOST_REGISTRY_NAME = 'docker-registry.local'
DOCKER_HOST_REGISTRY_PORT = 5000
DOCKER_IMAGE_MAYAN_NAME = 'mayanedms/mayanedms'
DOCKER_IMAGE_MAYAN_TAG = 's4.10'
DOCKER_CLI_IMAGE_VERSION = 'docker:28.2.2-cli'
DOCKER_DIND_IMAGE_VERSION = 'docker:28.2.2-dind'
DOCKER_ELASTIC_IMAGE_NAME = 'elasticsearch'
DOCKER_ELASTIC_IMAGE_TAG = '9.1.3'
DOCKER_LINUX_IMAGE_NAME = 'debian'
DOCKER_LINUX_IMAGE_TAG = '12.11-slim'
DOCKER_MYSQL_IMAGE_NAME = 'mariadb'
DOCKER_MYSQL_IMAGE_TAG = '11.8.3'
DOCKER_ORACLE_IMAGE_VERSION = 'wnameless/oracle-xe-11g-r2'
DOCKER_POSTGRESQL_IMAGE_NAME = 'postgres'
DOCKER_POSTGRESQL_IMAGE_TAG = '15.14'
DOCKER_POSTGRESQL_MAX_CONNECTIONS = 150
DOCKER_PYTHON_IMAGE_VERSION = 'python:3.13.7-slim'
DOCKER_RABBITMQ_IMAGE_NAME = 'rabbitmq'
DOCKER_RABBITMQ_IMAGE_TAG = '4.1.4-management'
DOCKER_REDIS_IMAGE_NAME = 'redis'
DOCKER_REDIS_IMAGE_TAG = '7.4.5'
DOCKER_TRAEFIK_IMAGE_NAME = 'traefik'
DOCKER_TRAEFIK_IMAGE_TAG = 'v2.11.29'
DOCKER_USER_UID = 1000
DOCKER_USER_GID = 1000
FORGE_DOCKER_BASE_IMAGE_NAME = 'ubuntu'
FORGE_DOCKER_BASE_IMAGE_TAG = 'noble-20250910'
FORGE_DOCKER_COMPOSE_PROJECT_NAME = 'mayanforge'
FORGE_DOCKER_IMAGE_NAME = 'mayanedms/mayanforge'
FORGE_PYTHON_PACKAGES = 'setuptools==80.9.0 fancycompleter==0.11.1'
FORGE_PYTHON_VERSION = '3.13'
FORGE_TRANSIFEX_VERSION = 'v1.6.17'
GITLAB_CI_BRANCH_BUILDS_DOCKER = 'ci/builds/docker'
GITLAB_CI_BRANCH_BUILDS_DOCUMENTATION = 'ci/builds/documentation'
GITLAB_CI_BRANCH_BUILDS_PYTHON = 'ci/builds/python'
GITLAB_CI_BRANCH_DEPLOYMENTS_DEMO = 'ci/deployments/demo'
GITLAB_CI_BRANCH_DEPLOYMENTS_STAGING = 'ci/deployments/staging'
GITLAB_CI_BRANCH_RELEASES_ALL_MAJOR = 'ci/releases/all/major'
GITLAB_CI_BRANCH_RELEASES_ALL_MINOR = 'ci/releases/all/minor'
GITLAB_CI_BRANCH_RELEASES_DOCKER_MAJOR = 'ci/releases/docker/major'
GITLAB_CI_BRANCH_RELEASES_DOCKER_MINOR = 'ci/releases/docker/minor'
GITLAB_CI_BRANCH_RELEASES_DOCUMENTATION = 'ci/releases/documentation'
GITLAB_CI_BRANCH_RELEASES_NIGHTLY = 'ci/releases/nightly'
GITLAB_CI_BRANCH_RELEASES_PYTHON_MAJOR = 'ci/releases/python/major'
GITLAB_CI_BRANCH_RELEASES_PYTHON_MINOR = 'ci/releases/python/minor'
GITLAB_CI_BRANCH_RELEASES_STAGING = 'ci/releases/staging'
GITLAB_CI_BRANCH_RELEASES_TESTING = 'ci/releases/testing'
GITLAB_CI_BRANCH_TESTS_ALL = 'ci/tests/all'
GITLAB_CI_BRANCH_TESTS_DOCKER = 'ci/tests/docker'
GITLAB_CI_BRANCH_TESTS_PYTHON_ALL = 'ci/tests/python/all'
GITLAB_CI_BRANCH_TESTS_PYTHON_BASE = 'ci/tests/python/base'
GITLAB_CI_BRANCH_TESTS_PYTHON_UPGRADE = 'ci/tests/python/upgrade'
GITLAB_CI_BRANCH_TRY_STAGING = 'ci/try/staging'
GITLAB_CI_REMOTE_NAME = 'origin'
GUNICORN_LIMIT_REQUEST_LINE = 4094
GUNICORN_MAX_REQUESTS = 500
GUNICORN_REQUESTS_JITTER = 50
GUNICORN_TIMEOUT = 120
GUNICORN_WORKER_CLASS = 'gevent'
GUNICORN_WORKERS = 3
LINUX_PACKAGES_DEBIAN_BASE = 'ca-certificates clamav file fuse ghostscript git gnupg1 graphviz libarchive-zip-perl libfile-mimeinfo-perl libimage-exiftool-perl libmariadb3 libpq5 libreoffice-calc-nogui libreoffice-draw-nogui libreoffice-impress-nogui libreoffice-math-nogui libreoffice-writer-nogui poppler-utils sane-utils sudo supervisor tesseract-ocr'
LINUX_PACKAGES_DEBIAN_BUILD = 'g++ gcc libffi-dev libjpeg-dev libldap2-dev libpng-dev libsasl2-dev libssl-dev libtiff-dev make pkg-config zlib1g-dev'
LINUX_PACKAGES_DEBIAN_DOCUMENTATION = 'enchant-2 graphviz libcairo2'
LINUX_PACKAGES_DEBIAN_FORGE = 'bash-completion curl gettext less software-properties-common squid-deb-proxy-client vim virtualenvwrapper'
LINUX_PACKAGES_DEBIAN_MYSQL = 'default-libmysqlclient-dev'
LINUX_PACKAGES_DEBIAN_POSTGRESQL = 'libpq-dev'
LINUX_PACKAGES_DEBIAN_PUSH = 'openssh-client rsync'
LINUX_PACKAGES_DEBIAN_PYTHON = 'make python3-dev python3-venv'
LINUX_PACKAGES_DEBIAN_TEST = 'tesseract-ocr-deu'
LINUX_PACKAGES_ALPINE_BUILD = 'cairo gcc libffi-dev libjpeg-turbo-dev libmagic make musl-dev zlib-dev'
LINUX_PACKAGES_ALPINE_PYTHON = 'python3 python3-dev py3-packaging'
LOCAL_VERSION = ''
PYTHON_AMQP_VERSION = '5.3.1'
PYTHON_MYSQL_VERSION = '2.2.7'
PYTHON_ORACLE_VERSION = '8.1.0'
PYTHON_PIP_VERSION = '25.2'
PYTHON_PSYCOPG_VERSION = '3.2.9'
PYTHON_PSYCOPG_VERSION_PREVIOUS = '3.1.20'
PYTHON_PSUTIL_VERSION = '6.1.1'
PYTHON_REDIS_VERSION = '6.4.0'
PYTHON_SETUPTOOLS_VERSION = '80.9.0'
PYTHON_WHEEL_VERSION = '0.45.1'
SOURCE_CODE_REPOSITORY = 'https://gitlab.com/mayan-edms/mayan-edms/'
SOURCE_CODE_GIT = 'https://gitlab.com/mayan-edms/mayan-edms.git'
SOURCE_CODE_ISSUES = 'https://gitlab.com/mayan-edms/mayan-edms/issues/'
SECRET_KEY_FILENAME = 'SECRET_KEY'
SYSTEM_DIR = 'system'
SUPERVISOR_CONFIGURATION_DIRECTORY = '/etc/supervisor/conf.d/'
SUPERVISOR_CONFIGURATION_FILENAME = 'mayan-edms.conf'
MAYAN_WORKER_A_CONCURRENCY = 4
MAYAN_WORKER_A_MAX_MEMORY_PER_CHILD = 500000
MAYAN_WORKER_A_MAX_TASKS_PER_CHILD = 4000
MAYAN_WORKER_A_NICE_LEVEL = 0
MAYAN_WORKER_B_CONCURRENCY = 4
MAYAN_WORKER_B_MAX_MEMORY_PER_CHILD = 500000
MAYAN_WORKER_B_MAX_TASKS_PER_CHILD = 4000
MAYAN_WORKER_B_NICE_LEVEL = 2
MAYAN_WORKER_C_CONCURRENCY = 4
MAYAN_WORKER_C_MAX_MEMORY_PER_CHILD = 500000
MAYAN_WORKER_C_MAX_TASKS_PER_CHILD = 1000
MAYAN_WORKER_C_NICE_LEVEL = 10
MAYAN_WORKER_D_CONCURRENCY = 1
MAYAN_WORKER_D_MAX_MEMORY_PER_CHILD = 500000
MAYAN_WORKER_D_MAX_TASKS_PER_CHILD = 500
MAYAN_WORKER_D_NICE_LEVEL = 15
MAYAN_WORKER_E_CONCURRENCY = 4
MAYAN_WORKER_E_MAX_MEMORY_PER_CHILD = 500000
MAYAN_WORKER_E_MAX_TASKS_PER_CHILD = 20000
MAYAN_WORKER_E_NICE_LEVEL = 2



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Document exports'), name='document_exports'
)

permission_document_version_export = namespace.add_permission(
    label=_(message='Export document versions'),
    name='document_version_export'
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.app_manager.apps import MayanAppConfig
from mayan.apps.common.menus import menu_object, menu_secondary, menu_tools
from mayan.apps.forms import column_widgets
from mayan.apps.navigation.source_columns import SourceColumn

from .classes import ErrorLogDomain
from .links import (
    link_global_error_log_partition_entry_list,
    link_object_error_log_entry_list_clear, link_object_error_log_entry_delete
)
from .literals import DEFAULT_ERROR_LOG_DOMAIN_NAME
from .mixins import LoggingAppConfigMixin


class LoggingApp(LoggingAppConfigMixin, MayanAppConfig):
    app_namespace = 'logging'
    app_url = 'logging'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.logging'
    verbose_name = _(message='Logging')

    def ready(self, *args, **kwargs):
        super().ready(*args, **kwargs)

        GlobalErrorLogPartitionEntry = self.get_model(
            model_name='GlobalErrorLogPartitionEntry'
        )
        ErrorLogPartition = self.get_model(
            model_name='ErrorLogPartition'
        )
        ErrorLogPartitionEntry = self.get_model(
            model_name='ErrorLogPartitionEntry'
        )

        ErrorLogDomain(
            name=DEFAULT_ERROR_LOG_DOMAIN_NAME, label=_(message='System')
        )

        ModelPermission.register_inheritance(
            model=ErrorLogPartitionEntry, related='error_log_partition'
        )
        ModelPermission.register_inheritance(
            model=ErrorLogPartition, related='content_object'
        )

        SourceColumn(
            attribute='error_log_partition__name', is_sortable=True,
            source=GlobalErrorLogPartitionEntry
        )
        SourceColumn(
            attribute='get_object', source=GlobalErrorLogPartitionEntry,
            widget=column_widgets.ObjectLinkWidget
        )

        SourceColumn(
            attribute='datetime', is_identifier=True, is_sortable=True,
            source=ErrorLogPartitionEntry
        )
        SourceColumn(
            attribute='get_domain_label', label=_(message='Domain'),
            source=ErrorLogPartitionEntry
        )
        SourceColumn(
            attribute='text', include_label=True, is_sortable=True,
            source=ErrorLogPartitionEntry
        )

        menu_object.bind_links(
            links=(link_object_error_log_entry_delete,), sources=(
                ErrorLogPartitionEntry,
            )
        )
        menu_secondary.bind_links(
            links=(link_object_error_log_entry_list_clear,), sources=(
                'logging:object_error_log_entry_list',
            )
        )
        menu_tools.bind_links(
            links=(link_global_error_log_partition_entry_list,),
        )



from django.utils.translation import gettext_lazy as _

from mayan.apps.rest_api import serializers
from mayan.apps.rest_api.relations import MultiKwargHyperlinkedIdentityField

from ..models.workflow_transition_field_models import WorkflowTransitionField


class WorkflowTransitionFieldSerializer(
    serializers.HyperlinkedModelSerializer
):
    url = MultiKwargHyperlinkedIdentityField(
        label=_(message='URL'), view_kwargs=(
            {
                'lookup_field': 'transition.workflow_id',
                'lookup_url_kwarg': 'workflow_template_id'
            },
            {
                'lookup_field': 'transition_id',
                'lookup_url_kwarg': 'workflow_template_transition_id'
            },
            {
                'lookup_field': 'pk',
                'lookup_url_kwarg': 'workflow_template_transition_field_id'
            }
        ), view_name='rest_api:workflow-template-transition-field-detail'
    )
    workflow_template_url = MultiKwargHyperlinkedIdentityField(
        label=_(message='Workflow template URL'), view_kwargs=(
            {
                'lookup_field': 'transition__workflow_id',
                'lookup_url_kwarg': 'workflow_template_id'
            },
        ), view_name='rest_api:workflow-template-detail'
    )
    workflow_transition_id = serializers.IntegerField(
        label=_(message='Workflow transition ID'), read_only=True,
        source='transition_id'
    )
    workflow_transition_url = MultiKwargHyperlinkedIdentityField(
        label=_(message='Workflow transition URL'), view_kwargs=(
            {
                'lookup_field': 'transition.workflow_id',
                'lookup_url_kwarg': 'workflow_template_id'
            },
            {
                'lookup_field': 'transition_id',
                'lookup_url_kwarg': 'workflow_template_transition_id'
            }
        ), view_name='rest_api:workflow-template-transition-detail'
    )

    class Meta:
        fields = (
            'field_type', 'name', 'help_text', 'id', 'label', 'lookup',
            'required', 'url', 'widget', 'widget_kwargs',
            'workflow_template_url', 'workflow_transition_id',
            'workflow_transition_url'
        )
        model = WorkflowTransitionField
        read_only_fields = (
            'id', 'url', 'workflow_template_url', 'workflow_transition_id',
            'workflow_transition_url'
        )



import datetime
import unicodedata

import dateparser
import dateutil

from django.utils.translation import gettext_lazy as _

from .exceptions import DynamicSearchValueTransformationError
from .value_transformation_mixins import ValueTransformationTimezoneMixin


class ValueTransformation:
    label = None

    def __init__(self, value):
        self.value = value

    def get_label(self):
        return getattr(self, 'label') or self.__class__

    def execute(self):
        try:
            return self._execute()
        except (AttributeError, TypeError, ValueError) as exception:
            """
            Encapsulate conversion exception into a custom exception at the
            last possible moment to avoid hiding local code errors.
            """
            raise DynamicSearchValueTransformationError(
                'Unable apply transformation `{}` to value `{}`.'.format(
                    self.get_label(), self.value
                )
            ) from exception


class ValueTransformationAccentReplace(ValueTransformation):
    label = _(message='Replace accents')

    def _execute(self):
        if self.value is not None:
            return ''.join(
                letter for letter in unicodedata.normalize('NFD', self.value) if unicodedata.category(letter) != 'Mn'
            )
        else:
            return None


class ValueTransformationAtReplace(ValueTransformation):
    label = _(message='@ sign replace')

    def _execute(self):
        if self.value is not None:
            return self.value.replace('@', '_at_')
        else:
            return None


class ValueTransformationCasefold(ValueTransformation):
    label = _(message='Case fold')

    def _execute(self):
        if self.value is not None:
            return self.value.casefold()
        else:
            return None


class ValueTransformationHyphenReplace(ValueTransformation):
    label = _(message='Hyphen replace')

    def _execute(self):
        if self.value is not None:
            return self.value.replace('-', '_')
        else:
            return None


class ValueTransformationHyphenStrip(ValueTransformation):
    label = _(message='Hyphen replace')

    def _execute(self):
        if self.value is not None:
            return self.value.replace('-', '')
        else:
            return None


class ValueTransformationToBoolean(ValueTransformation):
    label = _(message='To boolean')

    def _execute(self):
        if self.value is not None:
            if self.value.lower() == 'true':
                return True
            elif self.value.lower() == 'false':
                return False
            else:
                return None


class ValueTransformationToDateTime(
    ValueTransformationTimezoneMixin, ValueTransformation
):
    label = _(message='To date time')

    def _execute_(self):
        if self.value is not None:
            try:
                datetime_default = datetime.datetime(
                    year=datetime.MINYEAR, month=1, day=1, hour=0,
                    minute=0, tzinfo=datetime.timezone.utc
                )

                result = dateutil.parser.parse(
                    default=datetime_default, timestr=self.value
                )
                return result.replace(microsecond=0)
            except dateutil.parser.ParserError:
                result = dateparser.parse(date_string=self.value)
                if result is None:
                    raise ValueError
                else:
                    return result.replace(microsecond=0)
        else:
            return None


class ValueTransformationToDateTimeISOFormat(
    ValueTransformationTimezoneMixin, ValueTransformation
):
    label = _(message='Date time ISO format')

    def _execute_(self):
        if self.value is not None:
            value = self.value.replace(microsecond=0)

            return value.isoformat()
        else:
            return None


class ValueTransformationToDateTimeSimpleFormat(
    ValueTransformationTimezoneMixin, ValueTransformation
):
    label = _(message='Date time simple format')

    def _execute_(self):
        if self.value is not None:
            return self.value.strftime('%Y%m%d%H%M%S')
        else:
            return None


class ValueTransformationToDateTimeTimestamp(
    ValueTransformationTimezoneMixin, ValueTransformation
):
    label = _(message='Date time timestamp format')

    def _execute_(self):
        if self.value is not None:
            # Elasticsearch works with decimal seconds for exact match
            # but not for comparison (greater than, etc) matches.
            # Typecasting to integer works for all use cases for all
            # backends.
            return int(
                self.value.timestamp() * 1000
            )
        else:
            return None


class ValueTransformationToInteger(ValueTransformation):
    label = _(message='To integer')

    def _execute(self):
        if self.value is not None:
            return int(self.value)
        else:
            return None


class ValueTransformationToString(ValueTransformation):
    label = _(message='To string')

    def _execute(self):
        if self.value is not None:
            return str(self.value)
        else:
            return None



from django.apps import apps

from mayan.apps.document_indexing.tasks import (
    task_index_instance_document_add
)

from .literals import STORAGE_NAME_WORKFLOW_CACHE
from .settings import setting_workflow_image_cache_maximum_size
from .tasks import task_launch_all_workflow_for


def handler_create_workflow_image_cache(sender, **kwargs):
    Cache = apps.get_model(app_label='file_caching', model_name='Cache')
    Cache.objects.update_or_create(
        defaults={
            'maximum_size': setting_workflow_image_cache_maximum_size.value,
        }, defined_storage_name=STORAGE_NAME_WORKFLOW_CACHE,
    )


def handler_launch_workflow_on_create(sender, instance, created, **kwargs):
    if created:
        task_launch_all_workflow_for.apply_async(
            kwargs={'document_id': instance.pk}
        )


def handler_launch_workflow_on_type_change(sender, instance, **kwargs):
    task_launch_all_workflow_for.apply_async(
        kwargs={'document_id': instance.pk}
    )


def handler_transition_trigger(sender, **kwargs):

    WorkflowTransitionTriggerEvent = apps.get_model(
        app_label='document_states',
        model_name='WorkflowTransitionTriggerEvent'
    )

    action = kwargs['instance']

    WorkflowTransitionTriggerEvent.objects.check_triggers(action=action)


# Indexing, workflow template


def handler_workflow_template_post_edit(sender, **kwargs):
    if not kwargs.get('created', False):
        for workflow_instance in kwargs['instance'].instances.all():
            task_index_instance_document_add.apply_async(
                kwargs={'document_id': workflow_instance.document.pk}
            )


# Indexing, workflow state


def handler_workflow_template_state_post_edit(sender, **kwargs):
    if not kwargs.get('created', False):
        for workflow_instance in kwargs['instance'].workflow.instances.all():
            task_index_instance_document_add.apply_async(
                kwargs={'document_id': workflow_instance.document.pk}
            )


def handler_workflow_template_state_pre_delete(sender, **kwargs):
    for workflow_instance in kwargs['instance'].workflow.instances.all():
        # Remove each of the documents.
        # Trigger the remove event for each document so they can be
        # reindexed.
        workflow_instance.delete()
        task_index_instance_document_add.apply_async(
            kwargs={'document_id': workflow_instance.document.pk}
        )


# Indexing, workflow template


def handler_workflow_template_transition_post_edit(sender, **kwargs):
    if not kwargs.get('created', False):
        for workflow_instance in kwargs['instance'].workflow.instances.all():
            task_index_instance_document_add.apply_async(
                kwargs={'document_id': workflow_instance.document.pk}
            )


def handler_workflow_template_transition_pre_delete(sender, **kwargs):
    for workflow_instance in kwargs['instance'].workflow.instances.all():
        # Remove each of the documents.
        # Trigger the remove event for each document so they can be
        # reindexed.
        workflow_instance.delete()
        task_index_instance_document_add.apply_async(
            kwargs={'document_id': workflow_instance.document.pk}
        )



from mayan.apps.documents.api_views.api_view_mixins import (
    ParentObjectDocumentTypeAPIViewMixin
)
from mayan.apps.rest_api import generics

from ..permissions import permission_document_type_file_metadata_setup
from ..serializers.document_type_serializers import (
    DocumentTypeDriverConfigurationSerializer
)


class APIDocumentTypeFileMetadataDriverConfigurationDetailView(
    ParentObjectDocumentTypeAPIViewMixin, generics.RetrieveUpdateAPIView
):
    """
    get: Return the selected document type file metadata settings.
    patch: Set the selected document type file metadata settings.
    put: Set the selected document type file metadata settings.
    """
    lookup_url_kwarg = 'driver_id'
    mayan_object_permission_map = {
        'GET': permission_document_type_file_metadata_setup,
        'PATCH': permission_document_type_file_metadata_setup,
        'PUT': permission_document_type_file_metadata_setup
    }
    serializer_class = DocumentTypeDriverConfigurationSerializer

    def get_source_queryset(self):
        document_type = self.get_document_type()
        return document_type.file_metadata_driver_configurations.all()


class APIDocumentTypeFileMetadataDriverConfigurationListView(
    ParentObjectDocumentTypeAPIViewMixin, generics.ListAPIView
):
    """
    get: Return the list of document type file metadata settings.
    """
    lookup_url_kwarg = 'document_type_id'
    serializer_class = DocumentTypeDriverConfigurationSerializer

    def get_source_queryset(self):
        document_type = self.get_document_type(
            permission=permission_document_type_file_metadata_setup
        )
        return document_type.file_metadata_driver_configurations.all()



from pytz import common_timezones

from django.conf import settings
from django.utils import translation


def get_language_option_list():
    dict_languages = dict(settings.LANGUAGES)

    result = list(
        dict_languages.keys()
    )

    return result


def get_language_choices():
    return sorted(
        settings.LANGUAGES, key=lambda entry: entry[1]
    )


def to_language(promise, language):
    current_language = translation.get_language()
    translation.activate(language=language)
    result = str(promise)
    translation.activate(language=current_language)

    return result


def get_timezone_option_list():
    """
    Compatibility function.
    """
    return common_timezones


def get_timezone_choices():
    return zip(
        get_timezone_option_list(), get_timezone_option_list()
    )



import ollama

from django.utils.translation import gettext_lazy as _

from mayan.apps.common.serialization import yaml_load
from mayan.apps.common.utils import flatten_map
from mayan.apps.file_metadata.classes import FileMetadataDriver

from .literals import DEFAULT_TIMEOUT


class FileMetadataDriverOllamaChat(FileMetadataDriver):
    argument_name_list = ('host', 'messages', 'model', 'timeout')
    description = _(message='Analyze content using Ollama.')
    enabled = False
    internal_name = 'ollama_chat'
    label = _(message='Ollama Chat AI driver')
    mime_type_list = ('*',)

    @classmethod
    def get_argument_values_from_settings(cls):
        result = {'timeout': DEFAULT_TIMEOUT}

        setting_arguments = super().get_argument_values_from_settings()

        if setting_arguments:
            result.update(setting_arguments)

        return result

    def __init__(self, host, model, messages, timeout, **kwargs):
        super().__init__(**kwargs)

        self.host = host
        self.messages = yaml_load(stream=messages)
        self.model = model
        self.timeout = int(timeout)

    def _process(self, document_file):
        result = {}

        client = ollama.Client(host=self.host, timeout=self.timeout)

        response = client.chat(model=self.model, messages=self.messages)

        try:
            # Handle undocumented backwards incompatible change in Ollama.
            response.items()
        except AttributeError:
            dictionary = response.model_dump()
        else:
            dictionary = response

        flatten_map(dictionary=dictionary, result=result)

        return result



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('tags', '0002_tag_selection')
    ]

    operations = [
        migrations.RemoveField(
            model_name='tag',
            name='color'
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0022_auto_20150715_0258'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='label',
            field=models.CharField(
                default='', max_length=255, blank=True,
                help_text='The name of the document', verbose_name='Label',
                db_index=True
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='document',
            name='language',
            field=models.CharField(
                default=b'eng', max_length=8, verbose_name='Language',
                blank=True, choices=[
                    [b'aar', b'Afar'], [b'abk', b'Abkhazian'], [b'ace', b'Achinese'], [b'ach', b'Acoli'], [b'ada', b'Adangme'], [b'ady', b'Adyghe; Adygei'], [b'afa', b'Afro-Asiatic languages'], [b'afh', b'Afrihili'], [b'afr', b'Afrikaans'], [b'ain', b'Ainu'], [b'aka', b'Akan'], [b'akk', b'Akkadian'], [b'alb', b'Albanian'], [b'ale', b'Aleut'], [b'alg', b'Algonquian languages'], [b'alt', b'Southern Altai'], [b'amh', b'Amharic'], [b'ang', b'English, Old (ca. 450-1100)'], [b'anp', b'Angika'], [b'apa', b'Apache languages'], [b'ara', b'Arabic'], [b'arc', b'Official Aramaic (700-300 BCE); Imperial Aramaic (700-300 BCE)'], [b'arg', b'Aragonese'], [b'arm', b'Armenian'], [b'arn', b'Mapudungun; Mapuche'], [b'arp', b'Arapaho'], [b'art', b'Artificial languages'], [b'arw', b'Arawak'], [b'asm', b'Assamese'], [b'ast', b'Asturian; Bable; Leonese; Asturleonese'], [b'ath', b'Athapascan languages'], [b'aus', b'Australian languages'], [b'ava', b'Avaric'], [b'ave', b'Avestan'], [b'awa', b'Awadhi'], [b'aym', b'Aymara'], [b'aze', b'Azerbaijani'], [b'bad', b'Banda languages'], [b'bai', b'Bamileke languages'], [b'bak', b'Bashkir'], [b'bal', b'Baluchi'], [b'bam', b'Bambara'], [b'ban', b'Balinese'], [b'baq', b'Basque'], [b'bas', b'Basa'], [b'bat', b'Baltic languages'], [b'bej', b'Beja; Bedawiyet'], [b'bel', b'Belarusian'], [b'bem', b'Bemba'], [b'ben', b'Bengali'], [b'ber', b'Berber languages'], [b'bho', b'Bhojpuri'], [b'bih', b'Bihari languages'], [b'bik', b'Bikol'], [b'bin', b'Bini; Edo'], [b'bis', b'Bislama'], [b'bla', b'Siksika'], [b'bnt', b'Bantu languages'], [b'bos', b'Bosnian'], [b'bra', b'Braj'], [b'bre', b'Breton'], [b'btk', b'Batak languages'], [b'bua', b'Buriat'], [b'bug', b'Buginese'], [b'bul', b'Bulgarian'], [b'bur', b'Burmese'], [b'byn', b'Blin; Bilin'], [b'cad', b'Caddo'], [b'cai', b'Central American Indian languages'], [b'car', b'Galibi Carib'], [b'cat', b'Catalan; Valencian'], [b'cau', b'Caucasian languages'], [b'ceb', b'Cebuano'], [b'cel', b'Celtic languages'], [b'cha', b'Chamorro'], [b'chb', b'Chibcha'], [b'che', b'Chechen'], [b'chg', b'Chagatai'], [b'chi', b'Chinese'], [b'chk', b'Chuukese'], [b'chm', b'Mari'], [b'chn', b'Chinook jargon'], [b'cho', b'Choctaw'], [b'chp', b'Chipewyan; Dene Suline'], [b'chr', b'Cherokee'], [b'chu', b'Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic'], [b'chv', b'Chuvash'], [b'chy', b'Cheyenne'], [b'cmc', b'Chamic languages'], [b'cop', b'Coptic'], [b'cor', b'Cornish'], [b'cos', b'Corsican'], [b'cpe', b'Creoles and pidgins, English based'], [b'cpf', b'Creoles and pidgins, French-based'], [b'cpp', b'Creoles and pidgins, Portuguese-based'], [b'cre', b'Cree'], [b'crh', b'Crimean Tatar; Crimean Turkish'], [b'crp', b'Creoles and pidgins'], [b'csb', b'Kashubian'], [b'cus', b'Cushitic languages'], [b'cze', b'Czech'], [b'dak', b'Dakota'], [b'dan', b'Danish'], [b'dar', b'Dargwa'], [b'day', b'Land Dayak languages'], [b'del', b'Delaware'], [b'den', b'Slave (Athapascan)'], [b'dgr', b'Dogrib'], [b'din', b'Dinka'], [b'div', b'Divehi; Dhivehi; Maldivian'], [b'doi', b'Dogri'], [b'dra', b'Dravidian languages'], [b'dsb', b'Lower Sorbian'], [b'dua', b'Duala'], [b'dum', b'Dutch, Middle (ca. 1050-1350)'], [b'dut', b'Dutch; Flemish'], [b'dyu', b'Dyula'], [b'dzo', b'Dzongkha'], [b'efi', b'Efik'], [b'egy', b'Egyptian (Ancient)'], [b'eka', b'Ekajuk'], [b'elx', b'Elamite'], [b'eng', b'English'], [b'enm', b'English, Middle (1100-1500)'], [b'epo', b'Esperanto'], [b'est', b'Estonian'], [b'ewe', b'Ewe'], [b'ewo', b'Ewondo'], [b'fan', b'Fang'], [b'fao', b'Faroese'], [b'fat', b'Fanti'], [b'fij', b'Fijian'], [b'fil', b'Filipino; Pilipino'], [b'fin', b'Finnish'], [b'fiu', b'Finno-Ugrian languages'], [b'fon', b'Fon'], [b'fre', b'French'], [b'frm', b'French, Middle (ca. 1400-1600)'], [b'fro', b'French, Old (842-ca. 1400)'], [b'frr', b'Northern Frisian'], [b'frs', b'Eastern Frisian'], [b'fry', b'Western Frisian'], [b'ful', b'Fulah'], [b'fur', b'Friulian'], [b'gaa', b'Ga'], [b'gay', b'Gayo'], [b'gba', b'Gbaya'], [b'gem', b'Germanic languages'], [b'geo', b'Georgian'], [b'ger', b'German'], [b'gez', b'Geez'], [b'gil', b'Gilbertese'], [b'gla', b'Gaelic; Scottish Gaelic'], [b'gle', b'Irish'], [b'glg', b'Galician'], [b'glv', b'Manx'], [b'gmh', b'German, Middle High (ca. 1050-1500)'], [b'goh', b'German, Old High (ca. 750-1050)'], [b'gon', b'Gondi'], [b'gor', b'Gorontalo'], [b'got', b'Gothic'], [b'grb', b'Grebo'], [b'grc', b'Greek, Ancient (to 1453)'], [b'gre', b'Greek, Modern (1453-)'], [b'grn', b'Guarani'], [b'gsw', b'Swiss German; Alemannic; Alsatian'], [b'guj', b'Gujarati'], [b'gwi', b"Gwich'in"], [b'hai', b'Haida'], [b'hat', b'Haitian; Haitian Creole'], [b'hau', b'Hausa'], [b'haw', b'Hawaiian'], [b'heb', b'Hebrew'], [b'her', b'Herero'], [b'hil', b'Hiligaynon'], [b'him', b'Himachali languages; Western Pahari languages'], [b'hin', b'Hindi'], [b'hit', b'Hittite'], [b'hmn', b'Hmong; Mong'], [b'hmo', b'Hiri Motu'], [b'hrv', b'Croatian'], [b'hsb', b'Upper Sorbian'], [b'hun', b'Hungarian'], [b'hup', b'Hupa'], [b'iba', b'Iban'], [b'ibo', b'Igbo'], [b'ice', b'Icelandic'], [b'ido', b'Ido'], [b'iii', b'Sichuan Yi; Nuosu'], [b'ijo', b'Ijo languages'], [b'iku', b'Inuktitut'], [b'ile', b'Interlingue; Occidental'], [b'ilo', b'Iloko'], [b'ina', b'Interlingua (International Auxiliary Language Association)'], [b'inc', b'Indic languages'], [b'ind', b'Indonesian'], [b'ine', b'Indo-European languages'], [b'inh', b'Ingush'], [b'ipk', b'Inupiaq'], [b'ira', b'Iranian languages'], [b'iro', b'Iroquoian languages'], [b'ita', b'Italian'], [b'jav', b'Javanese'], [b'jbo', b'Lojban'], [b'jpn', b'Japanese'], [b'jpr', b'Judeo-Persian'], [b'jrb', b'Judeo-Arabic'], [b'kaa', b'Kara-Kalpak'], [b'kab', b'Kabyle'], [b'kac', b'Kachin; Jingpho'], [b'kal', b'Kalaallisut; Greenlandic'], [b'kam', b'Kamba'], [b'kan', b'Kannada'], [b'kar', b'Karen languages'], [b'kas', b'Kashmiri'], [b'kau', b'Kanuri'], [b'kaw', b'Kawi'], [b'kaz', b'Kazakh'], [b'kbd', b'Kabardian'], [b'kha', b'Khasi'], [b'khi', b'Khoisan languages'], [b'khm', b'Central Khmer'], [b'kho', b'Khotanese;Sakan'], [b'kik', b'Kikuyu; Gikuyu'], [b'kin', b'Kinyarwanda'], [b'kir', b'Kirghiz; Kyrgyz'], [b'kmb', b'Kimbundu'], [b'kok', b'Konkani'], [b'kom', b'Komi'], [b'kon', b'Kongo'], [b'kor', b'Korean'], [b'kos', b'Kosraean'], [b'kpe', b'Kpelle'], [b'krc', b'Karachay-Balkar'], [b'krl', b'Karelian'], [b'kro', b'Kru languages'], [b'kru', b'Kurukh'], [b'kua', b'Kuanyama; Kwanyama'], [b'kum', b'Kumyk'], [b'kur', b'Kurdish'], [b'kut', b'Kutenai'], [b'lad', b'Ladino'], [b'lah', b'Lahnda'], [b'lam', b'Lamba'], [b'lao', b'Lao'], [b'lat', b'Latin'], [b'lav', b'Latvian'], [b'lez', b'Lezghian'], [b'lim', b'Limburgan; Limburger; Limburgish'], [b'lin', b'Lingala'], [b'lit', b'Lithuanian'], [b'lol', b'Mongo'], [b'loz', b'Lozi'], [b'ltz', b'Luxembourgish; Letzeburgesch'], [b'lua', b'Luba-Lulua'], [b'lub', b'Luba-Katanga'], [b'lug', b'Ganda'], [b'lui', b'Luiseno'], [b'lun', b'Lunda'], [b'luo', b'Luo (Kenya and Tanzania)'], [b'lus', b'Lushai'], [b'mac', b'Macedonian'], [b'mad', b'Madurese'], [b'mag', b'Magahi'], [b'mah', b'Marshallese'], [b'mai', b'Maithili'], [b'mak', b'Makasar'], [b'mal', b'Malayalam'], [b'man', b'Mandingo'], [b'mao', b'Maori'], [b'map', b'Austronesian languages'], [b'mar', b'Marathi'], [b'mas', b'Masai'], [b'may', b'Malay'], [b'mdf', b'Moksha'], [b'mdr', b'Mandar'], [b'men', b'Mende'], [b'mga', b'Irish, Middle (900-1200)'], [b'mic', b"Mi'kmaq; Micmac"], [b'min', b'Minangkabau'], [b'mis', b'Uncoded languages'], [b'mkh', b'Mon-Khmer languages'], [b'mlg', b'Malagasy'], [b'mlt', b'Maltese'], [b'mnc', b'Manchu'], [b'mni', b'Manipuri'], [b'mno', b'Manobo languages'], [b'moh', b'Mohawk'], [b'mol', b'Moldavian; Moldovan'], [b'mon', b'Mongolian'], [b'mos', b'Mossi'], [b'mul', b'Multiple languages'], [b'mun', b'Munda languages'], [b'mus', b'Creek'], [b'mwl', b'Mirandese'], [b'mwr', b'Marwari'], [b'myn', b'Mayan languages'], [b'myv', b'Erzya'], [b'nah', b'Nahuatl languages'], [b'nai', b'North American Indian languages'], [b'nap', b'Neapolitan'], [b'nau', b'Nauru'], [b'nav', b'Navajo; Navaho'], [b'nbl', b'Ndebele, South; South Ndebele'], [b'nde', b'Ndebele, North; North Ndebele'], [b'ndo', b'Ndonga'], [b'nds', b'Low German; Low Saxon; German, Low; Saxon, Low'], [b'nep', b'Nepali'], [b'new', b'Nepal Bhasa; Newari'], [b'nia', b'Nias'], [b'nic', b'Niger-Kordofanian languages'], [b'niu', b'Niuean'], [b'nno', b'Norwegian Nynorsk; Nynorsk, Norwegian'], [b'nob', 'Bokm\xe5l, Norwegian; Norwegian Bokm\xe5l'], [b'nog', b'Nogai'], [b'non', b'Norse, Old'], [b'nor', b'Norwegian'], [b'nqo', b"N'Ko"], [b'nso', b'Pedi; Sepedi; Northern Sotho'], [b'nub', b'Nubian languages'], [b'nwc', b'Classical Newari; Old Newari; Classical Nepal Bhasa'], [b'nya', b'Chichewa; Chewa; Nyanja'], [b'nym', b'Nyamwezi'], [b'nyn', b'Nyankole'], [b'nyo', b'Nyoro'], [b'nzi', b'Nzima'], [b'oci', b'Occitan (post 1500)'], [b'oji', b'Ojibwa'], [b'ori', b'Oriya'], [b'orm', b'Oromo'], [b'osa', b'Osage'], [b'oss', b'Ossetian; Ossetic'], [b'ota', b'Turkish, Ottoman (1500-1928)'], [b'oto', b'Otomian languages'], [b'paa', b'Papuan languages'], [b'pag', b'Pangasinan'], [b'pal', b'Pahlavi'], [b'pam', b'Pampanga; Kapampangan'], [b'pan', b'Panjabi; Punjabi'], [b'pap', b'Papiamento'], [b'pau', b'Palauan'], [b'peo', b'Persian, Old (ca. 600-400 B.C.)'], [b'per', b'Persian'], [b'phi', b'Philippine languages'], [b'phn', b'Phoenician'], [b'pli', b'Pali'], [b'pol', b'Polish'], [b'pon', b'Pohnpeian'], [b'por', b'Portuguese'], [b'pra', b'Prakrit languages'], [b'pro', 'Proven\xe7al, Old (to 1500); Occitan, Old (to 1500)'], [b'pus', b'Pushto; Pashto'], [b'qaa-qtz', b'Reserved for local use'], [b'que', b'Quechua'], [b'raj', b'Rajasthani'], [b'rap', b'Rapanui'], [b'rar', b'Rarotongan; Cook Islands Maori'], [b'roa', b'Romance languages'], [b'roh', b'Romansh'], [b'rom', b'Romany'], [b'rum', b'Romanian'], [b'run', b'Rundi'], [b'rup', b'Aromanian; Arumanian; Macedo-Romanian'], [b'rus', b'Russian'], [b'sad', b'Sandawe'], [b'sag', b'Sango'], [b'sah', b'Yakut'], [b'sai', b'South American Indian languages'], [b'sal', b'Salishan languages'], [b'sam', b'Samaritan Aramaic'], [b'san', b'Sanskrit'], [b'sas', b'Sasak'], [b'sat', b'Santali'], [b'scn', b'Sicilian'], [b'sco', b'Scots'], [b'sel', b'Selkup'], [b'sem', b'Semitic languages'], [b'sga', b'Irish, Old (to 900)'], [b'sgn', b'Sign Languages'], [b'shn', b'Shan'], [b'sid', b'Sidamo'], [b'sin', b'Sinhala; Sinhalese'], [b'sio', b'Siouan languages'], [b'sit', b'Sino-Tibetan languages'], [b'sla', b'Slavic languages'], [b'slo', b'Slovak'], [b'slv', b'Slovenian'], [b'sma', b'Southern Sami'], [b'sme', b'Northern Sami'], [b'smi', b'Sami languages'], [b'smj', b'Lule Sami'], [b'smn', b'Inari Sami'], [b'smo', b'Samoan'], [b'sms', b'Skolt Sami'], [b'sna', b'Shona'], [b'snd', b'Sindhi'], [b'snk', b'Soninke'], [b'sog', b'Sogdian'], [b'som', b'Somali'], [b'son', b'Songhai languages'], [b'sot', b'Sotho, Southern'], [b'spa', b'Spanish; Castilian'], [b'srd', b'Sardinian'], [b'srn', b'Sranan Tongo'], [b'srp', b'Serbian'], [b'srr', b'Serer'], [b'ssa', b'Nilo-Saharan languages'], [b'ssw', b'Swati'], [b'suk', b'Sukuma'], [b'sun', b'Sundanese'], [b'sus', b'Susu'], [b'sux', b'Sumerian'], [b'swa', b'Swahili'], [b'swe', b'Swedish'], [b'syc', b'Classical Syriac'], [b'syr', b'Syriac'], [b'tah', b'Tahitian'], [b'tai', b'Tai languages'], [b'tam', b'Tamil'], [b'tat', b'Tatar'], [b'tel', b'Telugu'], [b'tem', b'Timne'], [b'ter', b'Tereno'], [b'tet', b'Tetum'], [b'tgk', b'Tajik'], [b'tgl', b'Tagalog'], [b'tha', b'Thai'], [b'tib', b'Tibetan'], [b'tig', b'Tigre'], [b'tir', b'Tigrinya'], [b'tiv', b'Tiv'], [b'tkl', b'Tokelau'], [b'tlh', b'Klingon; tlhIngan-Hol'], [b'tli', b'Tlingit'], [b'tmh', b'Tamashek'], [b'tog', b'Tonga (Nyasa)'], [b'ton', b'Tonga (Tonga Islands)'], [b'tpi', b'Tok Pisin'], [b'tsi', b'Tsimshian'], [b'tsn', b'Tswana'], [b'tso', b'Tsonga'], [b'tuk', b'Turkmen'], [b'tum', b'Tumbuka'], [b'tup', b'Tupi languages'], [b'tur', b'Turkish'], [b'tut', b'Altaic languages'], [b'tvl', b'Tuvalu'], [b'twi', b'Twi'], [b'tyv', b'Tuvinian'], [b'udm', b'Udmurt'], [b'uga', b'Ugaritic'], [b'uig', b'Uighur; Uyghur'], [b'ukr', b'Ukrainian'], [b'umb', b'Umbundu'], [b'und', b'Undetermined'], [b'urd', b'Urdu'], [b'uzb', b'Uzbek'], [b'vai', b'Vai'], [b'ven', b'Venda'], [b'vie', b'Vietnamese'], [b'vol', 'Volap\xfck'], [b'vot', b'Votic'], [b'wak', b'Wakashan languages'], [b'wal', b'Wolaitta; Wolaytta'], [b'war', b'Waray'], [b'was', b'Washo'], [b'wel', b'Welsh'], [b'wen', b'Sorbian languages'], [b'wln', b'Walloon'], [b'wol', b'Wolof'], [b'xal', b'Kalmyk; Oirat'], [b'xho', b'Xhosa'], [b'yao', b'Yao'], [b'yap', b'Yapese'], [b'yid', b'Yiddish'], [b'yor', b'Yoruba'], [b'ypk', b'Yupik languages'], [b'zap', b'Zapotec'], [b'zbl', b'Blissymbols; Blissymbolics; Bliss'], [b'zen', b'Zenaga'], [b'zgh', b'Standard Moroccan Tamazight'], [b'zha', b'Zhuang; Chuang'], [b'znd', b'Zande languages'], [b'zul', b'Zulu'], [b'zun', b'Zuni'], [b'zxx', b'No linguistic content; Not applicable'], [b'zza', b'Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki']
                ]
            ),
            preserve_default=True,
        ),
    ]



from django.db import migrations


def code_document_latest_file_populate(apps, schema_editor):
    cursor_main = schema_editor.connection.create_cursor(
        name='document_file_latest'
    )
    cursor_document_file_latest_update = schema_editor.connection.cursor()

    query_main = '''
        SELECT
            {documents_document}.{id}, {documents_documentfile_latest}.{id}
        FROM (
            SELECT
                {documents_documentfile_timestamp_max}.{document_id}, {documents_documentfile}.{id}
            FROM {documents_documentfile}, (
                SELECT
                    {document_id}, {id}, MAX({timestamp}) as {timestamp_max}
                FROM
                    {documents_documentfile}
                GROUP BY
                    {document_id}, {id}
            ) AS {documents_documentfile_timestamp_max}
            WHERE {documents_documentfile}.{timestamp} = {documents_documentfile_timestamp_max}.{timestamp_max}
            AND {documents_documentfile}.{document_id} = {documents_documentfile_timestamp_max}.{document_id}
        ) as {documents_documentfile_latest}
        INNER JOIN
          {documents_document}
        ON
          {documents_document}.{id} = {documents_documentfile_latest}.{document_id}
    '''.format(
        documents_document=schema_editor.connection.ops.quote_name(
            name='documents_document'
        ),
        documents_documentfile=schema_editor.connection.ops.quote_name(
            name='documents_documentfile'
        ),
        documents_documentfile_latest=schema_editor.connection.ops.quote_name(
            name='documents_documentfile_latest'
        ),
        documents_documentfile_timestamp_max=schema_editor.connection.ops.quote_name(
            name='documents_documentfile_timestamp_max'
        ),
        document_id=schema_editor.connection.ops.quote_name(
            name='document_id'
        ),
        id=schema_editor.connection.ops.quote_name(name='id'),
        timestamp=schema_editor.connection.ops.quote_name(name='timestamp'),
        timestamp_max=schema_editor.connection.ops.quote_name(
            name='timestamp_max'
        )
    )

    cursor_main.execute(query_main)

    FETCH_SIZE = 100000

    quoted_documents_document = schema_editor.connection.ops.quote_name(
        name='documents_document'
    )
    quoted_file_latest_id = schema_editor.connection.ops.quote_name(
        name='file_latest_id'
    )
    quoted_id = schema_editor.connection.ops.quote_name(name='id')

    while True:
        rows = cursor_main.fetchmany(FETCH_SIZE)
        for row in rows:
            query = f'''
                UPDATE {quoted_documents_document}
                    SET {quoted_file_latest_id} = {row[1]}
                WHERE {quoted_id} = {row[0]};
            '''

            cursor_document_file_latest_update.execute(query)

        if not rows:
            break


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0085_document_file_latest')
    ]

    operations = [
        migrations.RunPython(
            code=code_document_latest_file_populate,
            reverse_code=migrations.RunPython.noop
        )
    ]



from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

from .literals import (
    DEFAULT_REST_API_DISABLE_LINKS, DEFAULT_REST_API_MAXIMUM_PAGE_SIZE,
    DEFAULT_REST_API_PAGE_SIZE
)

setting_namespace = setting_cluster.do_namespace_add(
    label=_(message='REST API'), name='rest_api', version='0001'
)

setting_disable_links = setting_namespace.do_setting_add(
    choices=('false', 'true'), default=DEFAULT_REST_API_DISABLE_LINKS,
    global_name='REST_API_DISABLE_LINKS', help_text=_(
        message='Disable the REST API links in the tools menu.'
    )
)
setting_maximum_page_size = setting_namespace.do_setting_add(
    default=DEFAULT_REST_API_MAXIMUM_PAGE_SIZE,
    global_name='REST_API_MAXIMUM_PAGE_SIZE', help_text=_(
        message='The maximum page size that can be requested.'
    )
)
setting_page_size = setting_namespace.do_setting_add(
    default=DEFAULT_REST_API_PAGE_SIZE,
    global_name='REST_API_PAGE_SIZE', help_text=_(
        message='The default page size if none is specified.'
    )
)



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0091_fix_documenttype_verbose_name'),
        ('file_metadata', '0012_add_unique_together')
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentTypeDriverConfiguration',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, help_text='Enable this driver to '
                        'process document files of the selected document '
                        'type.', verbose_name='Enabled'
                    )
                ),
                (
                    'document_type', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='file_metadata_driver_configurations',
                        to='documents.documenttype',
                        verbose_name='Document type'
                    )
                ),
                (
                    'stored_driver', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='document_type_configurations',
                        to='file_metadata.storeddriver',
                        verbose_name='Driver'
                    )
                )
            ],
            options={
                'verbose_name': 'Document type driver settings',
                'verbose_name_plural': 'Document type driver settings',
                'ordering': ('stored_driver',),
                'unique_together': {
                    ('document_type', 'stored_driver')
                }
            }
        )
    ]



import functools

from django.utils.translation import gettext_lazy as _

from .exceptions import DangerousTagError
from .settings import setting_templating_dangerous_tags_allow_list
from .utils import get_tag_name


def templating_dangerous_tag(reason=None):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            tag_name = get_tag_name(tag=func)
            allowed_list = setting_templating_dangerous_tags_allow_list.value.split(',')

            if tag_name in allowed_list:
                return func(*args, **kwargs)
            else:
                reason_message = reason or _(message='None provided.')
                message = _(
                    message='Filter or tag "%(name)s" is considered dangerous '
                    'and not allowed. Reason: %(reason)s Change setting '
                    '`%(setting)s` to allow it.'
                ) % {
                    'name': tag_name, 'reason': reason_message,
                    'setting': setting_templating_dangerous_tags_allow_list.global_name
                }
                raise DangerousTagError(message)

        wrapper._dangerous_template_tag = True
        return wrapper
    return decorator



import qrcode

from mayan.apps.forms import form_fields

from .widgets import Base64ImageWidget, ImageWidget


class ImageField(form_fields.Field):
    widget = ImageWidget

    def __init__(self, *args, **kwargs):
        self.image_alt_text = kwargs.pop('image_alt_text', '')
        super().__init__(*args, **kwargs)
        self.widget.attrs['alt'] = self.image_alt_text


class ReadOnlyImageField(form_fields.ImageField):
    def clean(self, data, initial=None):
        return ''


class QRCodeImageField(ReadOnlyImageField):
    widget = Base64ImageWidget

    def prepare_value(self, value):
        instance = qrcode.QRCode()
        instance.add_data(data=value)
        instance.make(fit=True)

        qrcode_image = instance.make_image()

        size = qrcode_image.height / 2

        self.widget.attrs['height'] = size
        self.widget.attrs['width'] = size

        return qrcode_image



import logging

from rest_framework import status
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response

from mayan.apps.acls.models import AccessControlList
from mayan.apps.rest_api import generics

from ..classes import DocumentFileAction
from ..models.document_models import Document
from ..models.document_type_models import DocumentType
from ..permissions import (
    permission_document_change_type, permission_document_create,
    permission_document_properties_edit, permission_document_trash,
    permission_document_view
)
from ..serializers.document_serializers import (
    DocumentFileActionSerializer, DocumentSerializer,
    DocumentChangeTypeSerializer, DocumentUploadSerializer
)
from ..tasks.document_tasks import task_document_move_to_trash

logger = logging.getLogger(name=__name__)


class APIDocumentDetailView(generics.RetrieveUpdateDestroyAPIView):
    """
    Returns the selected document details.
    delete: Move the selected document to the thrash.
    get: Return the details of the selected document.
    patch: Edit the properties of the selected document.
    put: Edit the properties of the selected document.
    """
    lookup_url_kwarg = 'document_id'
    mayan_object_permission_map = {
        'DELETE': permission_document_trash,
        'GET': permission_document_view,
        'PATCH': permission_document_properties_edit,
        'PUT': permission_document_properties_edit
    }
    serializer_class = DocumentSerializer
    source_queryset = Document.valid.all()

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()

        task_document_move_to_trash.apply_async(
            kwargs={
                'document_id': instance.pk, 'user_id': self.request.user.pk
            }
        )

        return Response(status=status.HTTP_202_ACCEPTED)

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class APIDocumentFileActionListView(generics.ListAPIView):
    """
    get: Returns a list of the available document file actions.
    """
    serializer_class = DocumentFileActionSerializer

    def get_serializer_context(self):
        return {
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }

    def get_source_queryset(self):
        return DocumentFileAction.get_all()


class APIDocumentListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the documents.
    post: Create a new document.
    """
    mayan_object_permission_map = {'GET': permission_document_view}
    serializer_class = DocumentSerializer
    source_queryset = Document.valid.all()

    def perform_create(self, serializer):
        queryset = DocumentType.objects.all()

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_create, queryset=queryset,
            user=self.request.user
        )

        serializer.validated_data['document_type'] = get_object_or_404(
            pk=serializer.validated_data['document_type_id'],
            queryset=queryset
        )
        super().perform_create(serializer=serializer)

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class APIDocumentChangeTypeView(generics.ObjectActionAPIView):
    """
    post: Change the type of the selected document.
    """
    lookup_url_kwarg = 'document_id'
    mayan_object_permission_map = {'POST': permission_document_change_type}
    serializer_class = DocumentChangeTypeSerializer
    source_queryset = Document.valid.all()

    def object_action(self, obj, request, serializer):
        document_type_id = serializer.validated_data['document_type_id']
        obj.document_type_change(
            document_type=document_type_id, user=self.request.user
        )


class APIDocumentUploadView(generics.CreateAPIView):
    """
    post: Create a new document and a new document file.
    """
    serializer_class = DocumentUploadSerializer

    def perform_create(self, serializer):
        queryset = DocumentType.objects.all()

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_create, queryset=queryset,
            user=self.request.user
        )

        serializer.validated_data['document_type'] = get_object_or_404(
            pk=serializer.validated_data['document_type_id'],
            queryset=queryset
        )
        super().perform_create(serializer=serializer)

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}



from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.forms.document_type_forms import (
    DocumentTypeFilteredSelectForm
)
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.views.generics import (
    FormView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..icons import (
    icon_document_type_file_metadata_driver_configuration_edit,
    icon_document_type_file_metadata_driver_configuration_list,
    icon_document_type_file_metadata_submit, icon_file_metadata_driver
)
from ..models import DocumentTypeDriverConfiguration
from ..permissions import (
    permission_document_type_file_metadata_setup,
    permission_file_metadata_submit
)

from .view_mixins import ViewMixinDynamicConfigurationFormClass


class DocumentTypeFileMetadataDriverConfigurationEditView(
    ExternalObjectViewMixin, ViewMixinDynamicConfigurationFormClass,
    SingleObjectEditView
):
    external_object_class = DocumentType
    external_object_permission = permission_document_type_file_metadata_setup
    external_object_pk_url_kwarg = 'document_type_id'
    # Use a slug because Django does not support `pk_field`.
    slug_field = 'stored_driver_id'
    slug_url_kwarg = 'stored_driver_id'
    view_icon = icon_document_type_file_metadata_driver_configuration_edit

    def get_extra_context(self):
        return {
            'document_type': self.external_object,
            'navigation_object_list': ('document_type', 'object'),
            'object': self.object,
            'title': _(
                message='Edit file metadata driver '
                '"%(file_metadata_driver)s" settings for document type: '
                '%(document_type)s'
            ) % {
                'document_type': self.external_object,
                'file_metadata_driver': self.object
            }
        }

    def get_post_action_redirect(self):
        return reverse(
            kwargs={'document_type_id': self.external_object.pk},
            viewname='file_metadata:document_type_file_metadata_driver_configuration_list'
        )

    def get_source_queryset(self):
        return DocumentTypeDriverConfiguration.valid.filter(
            document_type=self.external_object
        )


class DocumentTypeFileMetadataDriverConfigurationListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_class = DocumentType
    external_object_permission = permission_document_type_file_metadata_setup
    external_object_pk_url_kwarg = 'document_type_id'
    view_icon = icon_document_type_file_metadata_driver_configuration_list

    def get_extra_context(self):
        return {
            'document_type': self.external_object,
            'hide_object': True,
            'navigation_object_list': ('document_type',),
            'no_results_icon': icon_file_metadata_driver,
            'no_results_text': _(
                message='File metadata drivers extract embedded information '
                'from document files.'
            ),
            'no_results_title': _(
                message='No file metadata drivers available for this '
                'document type.'
            ),
            'title': _(
                message='File metadata driver configuration for document '
                'type "%s".'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return DocumentTypeDriverConfiguration.valid.filter(
            document_type=self.external_object
        )


class DocumentTypeFileMetadataSubmitView(FormView):
    extra_context = {
        'title': _(
            message='Submit all documents of a type for file metadata '
            'processing.'
        )
    }
    form_class = DocumentTypeFilteredSelectForm
    post_action_redirect = reverse_lazy(viewname='common:tools_list')
    view_icon = icon_document_type_file_metadata_submit

    def get_form_extra_kwargs(self):
        return {
            'allow_multiple': True,
            'permission': permission_file_metadata_submit,
            'user': self.request.user
        }

    def form_valid(self, form):
        queryset_documents = Document.valid.all()

        count = 0
        for document_type in form.cleaned_data['document_type']:
            for document in document_type.documents.filter(pk__in=queryset_documents.values('pk')):
                document.submit_for_file_metadata_processing(
                    user=self.request.user
                )
                count += 1

        messages.success(
            message=_(
                message='%(count)d documents added to the file metadata processing '
                'queue.'
            ) % {
                'count': count
            }, request=self.request
        )

        return HttpResponseRedirect(
            redirect_to=self.get_success_url()
        )



from django.utils.translation import gettext_lazy as _

from .events import event_file_metadata_document_file_submitted
from .tasks import task_document_file_metadata_process


def method_document_file_metadata_submit(self, user=None):
    latest_file = self.file_latest
    # Don't error out if document has no file.
    if latest_file:
        latest_file.submit_for_file_metadata_processing(user=user)


def method_document_file_metadata_submit_single(self, user=None):
    event_file_metadata_document_file_submitted.commit(
        action_object=self.document, actor=user, target=self
    )

    if user:
        user_id = user.pk
    else:
        user_id = None

    task_document_file_metadata_process.apply_async(
        kwargs={'document_file_id': self.pk, 'user_id': user_id}
    )


def method_get_document_file_file_metadata(self, dotted_name):
    parts = dotted_name.split('__', 1)

    if len(parts) < 2:
        return

    driver_internal_name = parts[0]
    attribute_internal_name = parts[1]

    try:
        document_driver = self.file_metadata_drivers.get(
            driver__internal_name=driver_internal_name
        )
    except self.file_metadata_drivers.model.DoesNotExist:
        return
    else:
        try:
            file_metadata_entry = document_driver.entries.get(
                internal_name=attribute_internal_name
            )
        except document_driver.entries.model.DoesNotExist:
            return
        else:
            return file_metadata_entry.value


method_get_document_file_file_metadata.help_text = _(
    message='Return the specified document file file metadata entry.'
)



from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion

import mayan.apps.views.model_mixins


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('views', '0001_initial')
    ]

    operations = [
        migrations.CreateModel(
            name='UserConfirmView',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'namespace', models.CharField(
                        db_index=True, max_length=200,
                        verbose_name='Namespace'
                    )
                ),
                (
                    'name', models.CharField(
                        db_index=True, help_text='Full name of the view '
                        'including the namespace.', max_length=200,
                        verbose_name='Name'
                    )
                ),
                (
                    'remember', models.BooleanField(
                        default=False, help_text='Remember the last '
                        'confirmation of the view.', verbose_name='Remember'
                    )
                ),
                (
                    'user', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='view_confirms',
                        to=settings.AUTH_USER_MODEL, verbose_name='User'
                    )
                )
            ],
            options={
                'verbose_name': 'User confirm view',
                'verbose_name_plural': 'User confirm view',
                'ordering': ('user__username', 'name'),
                'unique_together': {('user', 'name')}
            },
            bases=(
                mayan.apps.views.model_mixins.ModelMixinUserConfirmViewBusinessLogic,
                models.Model
            )
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('file_metadata', '0015_delete_documenttypesettings')
    ]

    operations = [
        migrations.AddField(
            model_name='storeddriver', name='exists',
            field=models.BooleanField(
                default=True, help_text='The class defined by this instance '
                'is valid and active.', verbose_name='Valid'
            )
        )
    ]



from django.utils.translation import gettext_lazy as _

from .classes import ErrorLog as ErrorLogProxy, ErrorLogDomain


class ErrorLogPartitionEntryBusinessLogicMixin:
    def get_domain_label(self):
        try:
            error_log_domain = ErrorLogDomain.get(name=self.domain_name)
        except KeyError:
            return _('Unknown domain name: %s' % self.domain_name)
        else:
            return error_log_domain.label

    def get_object(self):
        return self.error_log_partition.content_object

    get_object.short_description = _(message='Object')


class StoredErrorLogBusinessLogicMixin:
    @property
    def app_label(self):
        return self.proxy.app_config

    app_label.fget.short_description = _(message='App label')

    @property
    def proxy(self):
        return ErrorLogProxy.get(name=self.name)



from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('document_comments', '0006_auto_20210130_0658')
    ]

    operations = [
        migrations.AlterField(
            field=models.TextField(
                help_text='Actual text content of the comment.',
                verbose_name='Text'
            ), model_name='comment', name='text'
        ),
        migrations.AlterField(
            field=models.ForeignKey(
                editable=False, help_text='The user account that made the '
                'comment.', on_delete=django.db.models.deletion.CASCADE,
                related_name='comments', to=settings.AUTH_USER_MODEL,
                verbose_name='User'
            ), model_name='comment', name='user'
        )
    ]
